@@ -21,10 +21,6 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include <netinet/in.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -108,18 +104,9 @@ gboolean client_send_response(struct irc_client *c, int response, ...)
 	g_assert(response > 0);
 	
 	va_start(ap, response);
-	l = virc_parse_line(c->default_origin, ap);
+	l = virc_parse_response(c->default_origin, client_get_default_target(c), response, ap);
 	va_end(ap);
 
-	l->args = g_realloc(l->args, sizeof(char *) * (l->argc+4));
-	memmove(&l->args[2], &l->args[0], l->argc * sizeof(char *));
-
-	l->args[0] = g_strdup_printf("%03d", response);
-	l->args[1] = g_strdup(client_get_default_target(c));
-
-	l->argc+=2;
-	l->args[l->argc] = NULL;
-
 	ret = client_send_line(c, l);
 
 	free_line(l);
@@ -188,6 +175,7 @@ gboolean client_send_line(struct irc_client *c, const struct irc_line *l)
 
 	g_assert(c != NULL);
 	g_assert(l != NULL);
+
 	if (c->callbacks->process_to_client != NULL)
 		c->callbacks->process_to_client(c, l);
 
@@ -232,9 +220,8 @@ static void free_client(struct irc_client *c)
 	g_assert(c->connected == FALSE);
 	g_free(c->description);
 	free_network_state(c->state);
-	g_free(c->requested_nick);
-	g_free(c->requested_username);
-	g_free(c->requested_hostname);
+	free_login_details(c->login_details);
+	g_free(c->client_hostname);
 	free_irc_transport(c->transport);
 	g_free(c->default_origin);
 	if (c->callbacks->free_private_data)
@@ -269,27 +256,24 @@ static gboolean on_transport_receive_line(struct irc_transport *transport,
 {
 	struct irc_client *client = transport->userdata;
 
-	g_assert(l);
-
 	/* Silently drop empty messages */
-	if (l->argc == 0) {
+	if (l == NULL || l->argc == 0) {
 		return TRUE;
 	}
 
-	if (client->authorized) {
+	if (client->authenticated) {
 		return client->callbacks->process_from_client(client, l);
 	} else {
 		if (!process_from_pending_client(client, l)) {
 			return FALSE;
 		}
 
-		if (client->requested_nick != NULL &&
-			client->requested_username != NULL &&
-			client->requested_hostname != NULL && 
+		if (client->login_details->nick != NULL &&
+			client->login_details->username != NULL &&
 			client->state == NULL) {
-			client->state = network_state_init(client->requested_nick, 
-											   client->requested_username,
-											   client->requested_hostname);
+			client->state = network_state_init(client->login_details->nick, 
+											   client->login_details->username,
+											   client->client_hostname);
 		}
 
 		if (client->state != NULL) {
@@ -306,10 +290,9 @@ static gboolean on_transport_receive_line(struct irc_transport *transport,
 			pending_clients = g_list_remove(pending_clients, client);
 			client->network->clients = g_list_append(client->network->clients, client);
 
-			client->authorized = TRUE;
+			client->authenticated = TRUE;
 
 			client_log(LOG_INFO, client, "New client");
-
 		}
 	}
 	return TRUE;
@@ -318,8 +301,6 @@ static gboolean on_transport_receive_line(struct irc_transport *transport,
 static gboolean process_from_pending_client(struct irc_client *client, 
 											const struct irc_line *l)
 {
-	extern struct global *my_global;
-
 	if (!g_strcasecmp(l->args[0], "NICK")) {
 		if (l->argc < 2) {
 			client_send_response(client, ERR_NEEDMOREPARAMS,
@@ -327,7 +308,7 @@ static gboolean process_from_pending_client(struct irc_client *client,
 			return TRUE;
 		}
 
-		client->requested_nick = g_strdup(l->args[1]);
+		client->login_details->nick = g_strdup(l->args[1]);
 	} else if (!g_strcasecmp(l->args[0], "USER")) {
 
 		if (l->argc < 5) {
@@ -336,8 +317,10 @@ static gboolean process_from_pending_client(struct irc_client *client,
 			return TRUE;
 		}
 
-		client->requested_username = g_strdup(l->args[1]);
-		client->requested_hostname = g_strdup(l->args[2]);
+		client->login_details->username = g_strdup(l->args[1]);
+		client->login_details->mode = g_strdup(l->args[2]);
+		client->login_details->unused = g_strdup(l->args[3]);
+		client->login_details->realname = g_strdup(l->args[4]);
 	} else if (!g_strcasecmp(l->args[0], "PASS")) {
 		/* Silently drop... */
 	} else if (!g_strcasecmp(l->args[0], "CONNECT")) {
@@ -347,21 +330,7 @@ static gboolean process_from_pending_client(struct irc_client *client,
 			return TRUE;
 		}
 
-		client->network = network_ref(find_network_by_hostname(my_global, 
-				l->args[1], l->args[2]?atoi(l->args[2]):6667, my_global->config->create_implicit));
-
-		if (client->network == NULL) {
-			client_log(LOG_ERROR, client, 
-				"Unable to connect to network with name %s", 
-				l->args[1]);
-		}
-
-		if (client->network->connection.state == NETWORK_CONNECTION_STATE_NOT_CONNECTED) {
-			client_send_args(client, "NOTICE", 
-								client_get_default_target(client),
-								"Connecting to network", NULL);
-			connect_network(client->network);
-		}
+		client->network = client->callbacks->on_connect(client, l->args[1], l->args[2]?atoi(l->args[2]):6667);
 	} else {
 		client_send_response(client, ERR_NOTREGISTERED, 
 			"Register first", 
@@ -380,10 +349,18 @@ static gboolean process_from_pending_client(struct irc_client *client,
  */
 static gboolean client_ping(struct irc_client *client)
 {
+	const char *name;
 	g_assert(client != NULL);
 
 	client->last_ping = time(NULL);
-	client_send_args_ex(client, NULL, "PING", client->network->info->name, NULL);
+
+	if (client->state != NULL && client->state->info->name != NULL) {
+		name = client->state->info->name;
+	} else {
+		name = "PinglCtrlproxy";
+	}
+
+	client_send_args_ex(client, NULL, "PING", name, NULL);
 
 	return TRUE;
 }
@@ -397,6 +374,17 @@ const struct irc_transport_callbacks client_transport_callbacks = {
 	.hangup = irc_transport_disconnect,
 };
 
+void free_login_details(struct irc_login_details *details)
+{
+	g_free(details->nick);
+	g_free(details->password);
+	g_free(details->realname);
+	g_free(details->unused);
+	g_free(details->username);
+	g_free(details->mode);
+	g_free(details);
+}
+
 /* 
  * @param desc Description of the client
  */
@@ -411,6 +399,11 @@ struct irc_client *irc_client_new(struct irc_transport *transport, const char *d
 	g_assert(client != NULL);
 	client->references = 1;
 
+	client->login_details = g_new0(struct irc_login_details, 1);
+	client->client_hostname = transport_get_peer_hostname(transport);
+	if (client->client_hostname == NULL)
+		client->client_hostname = g_strdup("UNKNOWN");
+
 	client->default_origin = g_strdup(default_origin);
 	client->callbacks = callbacks;
 
@@ -637,7 +630,7 @@ void client_send_topic(struct irc_client *c, struct irc_channel_state *ch,
 	}
 
 	if (ch->topic_set_time != 0 && ch->topic_set_by != NULL) {
-		char *tmp = g_strdup_printf("%lu", ch->topic_set_time);
+		char *tmp = g_strdup_printf("%ld", ch->topic_set_time);
 		client_send_response(c, RPL_TOPICWHOTIME, ch->name, ch->topic_set_by, 
 							 tmp, NULL);
 		g_free(tmp);
@@ -659,8 +652,9 @@ void client_send_nameslist(struct irc_client *c,
 		struct channel_nick *n = (struct channel_nick *)nl->data;
 		char prefix;
 
+		g_assert(c->state != NULL && c->state->info != NULL);
 		if (n->modes != NULL) {
-			prefix = get_prefix_from_modes(ch->network->info, n->modes);
+			prefix = get_prefix_from_modes(c->state->info, n->modes);
 		} else {
 			prefix = 0;
 		}
@@ -723,9 +717,9 @@ void client_send_banlist(struct irc_client *client, struct irc_channel_state *ch
 	g_assert(channel);
 	g_assert(client);
 
-	for (gl = channel->banlist; gl; gl = gl->next)
+	for (gl = channel_mode_nicklist(channel, 'b'); gl; gl = gl->next)
 	{
-		struct banlist_entry *be = gl->data;
+		struct nicklist_entry *be = gl->data;
 		g_assert(be);
 		client_send_response(client, RPL_BANLIST, channel->name, be->hostmask, NULL);
 	}
@@ -737,9 +731,29 @@ void client_send_channel_mode(struct irc_client *c, struct irc_channel_state *ch
 {
 		char *mode;
 		mode = mode2string(ch->modes);
-		if (mode != NULL)
-			client_send_response(c, RPL_CHANNELMODEIS, ch->name, mode, NULL);
-		g_free(mode);
+		if (mode != NULL) {
+			struct irc_line *l;
+			int i, j = 0;
+			l = g_new0(struct irc_line, 1);
+			l->args = g_new0(char *, 260);
+			l->origin = g_strdup(c->default_origin);
+			l->args[0] = g_strdup_printf("%03d", RPL_CHANNELMODEIS);
+			l->args[1] = g_strdup(client_get_default_target(c));
+			l->args[2] = g_strdup(ch->name);
+			l->args[3] = mode;
+			for (i = 0; mode[i]; i++) {
+				if (network_chanmode_type(mode[i], c->state->info) == CHANMODE_OPT_SETTING && 
+					channel_mode_option(ch, mode[i])) {
+					l->args[4+j] = g_strdup(channel_mode_option(ch, mode[i]));
+					j++;
+				}
+				l->args[4+j] = NULL;
+			}
+			l->argc = 4+j;
+			client_send_line(c, l);
+			free_line(l);
+		}
+
 		if (ch->creation_time > 0) {
 			char time[20];
 			snprintf(time, sizeof(time), "%lu", ch->creation_time);
@@ -757,36 +771,41 @@ void client_send_luserchannels(struct irc_client *client, int num)
 
 }
 
-void clients_invalidate_state(GList *clients)
+void clients_send_netsplit(GList *clients, const char *lost_server)
 {
 	GList *gl;
 	for (gl = clients; gl; gl = gl->next) {
 		struct irc_client *c = gl->data;
-		client_invalidate_state(c);
+		client_send_netsplit(c, lost_server);
 	}
 }
 
-void client_invalidate_state(struct irc_client *c)
+void client_send_netsplit(struct irc_client *c, const char *lost_server)
 {
 	struct irc_network_state *s = c->state;
+	char *reason;
 	GList *gl;
 
-	/* Leave channels */
-	for (gl = s->channels; gl; gl = gl->next) {
-		struct irc_channel_state *ch = gl->data;
+	/* Spoof a netsplit */
+	reason = g_strdup_printf("%s %s", get_my_hostname(), lost_server);
 
-		client_send_args_ex(c, s->me.hostmask, "PART", ch->name, 
-							 "Network disconnected", NULL);
-	}
-
-	/* private queries quit */
+	/* all nicks known quit */
 	for (gl = s->nicks; gl; gl = gl->next) {
 		struct network_nick *gn = gl->data;
 
-		if (!gn->query || gn == &s->me) continue;
+		if (gn == &s->me) continue;
 
-		client_send_args_ex(c, gn->hostmask, "QUIT", "Network disconnected", NULL);
+		if (gn->hostmask == NULL) {
+			/* Make up a fake hostmask. The user hasn't seen the original hostmask 
+			 * of this nick anyway, otherwise we would've known it already. */
+			char *fake_hostmask = g_strdup_printf("%s!~UNKNOWN@UNKNOWN", gn->nick);
+			client_send_args_ex(c, fake_hostmask, "QUIT", reason, NULL);
+			g_free(fake_hostmask);
+		} else {
+			client_send_args_ex(c, gn->hostmask, "QUIT", reason, NULL);
+		}
 	}
+
+	g_free(reason);
 }
 
-
