@@ -38,29 +38,32 @@ static void state_log_helper(enum log_level l, void *userdata, const char *msg)
 
 static void server_send_login (struct irc_network *s) 
 {
-	struct network_config *nc = s->private_data;
+	struct irc_login_details *login_details = s->callbacks->get_login_details(s);
 	g_assert(s);
 
 	s->connection.state = NETWORK_CONNECTION_STATE_LOGIN_SENT;
 
 	network_log(LOG_TRACE, s, "Sending login details");
 
-	s->state = network_state_init(nc->nick, nc->username, 
+	s->external_state = network_state_init(login_details->nick, login_details->username, 
 								  get_my_hostname());
-	network_state_set_log_fn(s->state, state_log_helper, s);
-	s->linestack = new_linestack(s);
+	network_state_set_log_fn(s->external_state, state_log_helper, s);
+	s->linestack = new_linestack(s, s->global->config);
 	g_assert(s->linestack != NULL);
 
-	if (nc->type == NETWORK_TCP && 
-	   s->connection.data.tcp.current_server->password) { 
-		network_send_args(s, "PASS", 
-					s->connection.data.tcp.current_server->password, NULL);
-	} else if (nc->password) {
-		network_send_args(s, "PASS", nc->password, NULL);
+	if (login_details->password != NULL) {
+		network_send_args(s, "PASS", login_details->password, NULL);
 	}
-	network_send_args(s, "NICK", nc->nick, NULL);
-	network_send_args(s, "USER", nc->username, nc->username, 
-					  nc->name, nc->fullname, NULL);
+	g_assert(login_details->nick != NULL && strlen(login_details->nick) > 0);
+	network_send_args(s, "NICK", login_details->nick, NULL);
+	g_assert(login_details->username != NULL && strlen(login_details->username) > 0);
+	g_assert(login_details->mode != NULL && strlen(login_details->mode) > 0);
+	g_assert(login_details->unused != NULL && strlen(login_details->unused) > 0);
+	g_assert(login_details->realname != NULL && strlen(login_details->realname) > 0);
+	network_send_args(s, "USER", login_details->username, login_details->mode, 
+					  login_details->unused, login_details->realname, NULL);
+
+	free_login_details(login_details);
 }
 
 /**
@@ -195,8 +198,11 @@ static gboolean network_send_line_direct(struct irc_network *s, struct irc_clien
 		if (s->connection.data.virtual.ops == NULL) 
 			return FALSE;
 		return s->connection.data.virtual.ops->to_server(s, c, l);
-	} else 
+	} else {
+		if (s->connection.transport == NULL)
+			return FALSE;
 		return transport_send_line(s->connection.transport, l);
+	}
 }
 
 /**
@@ -217,8 +223,8 @@ gboolean network_send_line(struct irc_network *s, struct irc_client *c,
 	g_assert(s);
 	l = *ol;
 
-	if (l.origin == NULL && s->state != NULL) {
-		tmp = l.origin = g_strdup(s->state->me.hostmask);
+	if (l.origin == NULL && s->external_state != NULL) {
+		tmp = l.origin = g_strdup(s->external_state->me.hostmask);
 	}
 
 	if (l.origin != NULL) {
@@ -229,7 +235,7 @@ gboolean network_send_line(struct irc_network *s, struct irc_client *c,
 
 		run_log_filter(s, lc = linedup(&l), TO_SERVER); free_line(lc);
 		run_replication_filter(s, lc = linedup(&l), TO_SERVER); free_line(lc);
-		linestack_insert_line(s->linestack, ol, TO_SERVER, s->state);
+		linestack_insert_line(s->linestack, ol, TO_SERVER, s->external_state);
 	}
 
 	g_assert(l.args[0] != NULL);
@@ -274,7 +280,7 @@ gboolean virtual_network_recv_response(struct irc_network *n, int num, ...)
 	g_assert(nc->type == NETWORK_VIRTUAL);
 
 	va_start(ap, num);
-	l = virc_parse_line(n->info->name, ap);
+	l = virc_parse_line(n->name, ap);
 	va_end(ap);
 
 	l->args = g_realloc(l->args, sizeof(char *) * (l->argc+4));
@@ -282,8 +288,8 @@ gboolean virtual_network_recv_response(struct irc_network *n, int num, ...)
 
 	l->args[0] = g_strdup_printf("%03d", num);
 
-	if (n->state != NULL && n->state->me.nick != NULL) 
-		l->args[1] = g_strdup(n->state->me.nick);
+	if (n->external_state != NULL && n->external_state->me.nick != NULL) 
+		l->args[1] = g_strdup(n->external_state->me.nick);
 	else 
 		l->args[1] = g_strdup("*");
 
@@ -467,7 +473,8 @@ static gboolean connect_current_tcp_server(struct irc_network *s)
 	if (error) {
 		network_log(LOG_ERROR, s, "Unable to lookup %s:%s %s", 
 					cs->host, cs->port, gai_strerror(error));
-		freeaddrinfo(addrinfo);
+		if (addrinfo != NULL)
+			freeaddrinfo(addrinfo);
 		return FALSE;
 	}
 
@@ -623,16 +630,16 @@ static gboolean close_server(struct irc_network *n)
 
 	if (n->connection.state == NETWORK_CONNECTION_STATE_MOTD_RECVD) {
 		server_disconnected_hook_execute(n);
-		clients_invalidate_state(n->clients);
-		network_update_config(n->state, nc);
+		clients_send_netsplit(n->clients, n->external_state->info->server);
+		network_update_config(n->external_state, nc, n->global->config);
 	}
 
-	if (n->state) {
+	if (n->external_state) {
 		if (n->linestack != NULL)
 			free_linestack_context(n->linestack);
 		n->linestack = NULL;
-		free_network_state(n->state); 
-		n->state = NULL;
+		free_network_state(n->external_state); 
+		n->external_state = NULL;
 	}
 
 	g_assert(nc);
@@ -794,17 +801,45 @@ static gboolean connect_program(struct irc_network *s)
 
 	g_io_channel_unref(ioc);
 
-	if (s->info->name == NULL) {
+	if (s->name == NULL) {
 		if (strchr(nc->type_settings.program_location, '/')) {
-			s->info->name = g_strdup(strrchr(nc->type_settings.program_location, '/')+1);
+			s->name = g_strdup(strrchr(nc->type_settings.program_location, '/')+1);
 		} else {
-			s->info->name = g_strdup(nc->type_settings.program_location);
+			s->name = g_strdup(nc->type_settings.program_location);
 		}
 	}
 
 	return TRUE;
 }
 
+static gboolean connect_virtual(struct irc_network *s)
+{
+	struct irc_login_details *details;
+	struct network_config *nc = s->private_data;
+
+	s->connection.data.virtual.ops = g_hash_table_lookup(
+							virtual_network_ops, 
+							nc->type_settings.virtual_type);
+	if (!s->connection.data.virtual.ops) 
+		return FALSE;
+
+	details = s->callbacks->get_login_details(s);
+
+	s->external_state = network_state_init(details->nick, details->username, 
+								  get_my_hostname());
+
+	free_login_details(details);
+	s->external_state->userdata = s;
+	s->external_state->log = state_log_helper;
+	s->linestack = new_linestack(s, s->global->config);
+	s->connection.state = NETWORK_CONNECTION_STATE_MOTD_RECVD;
+
+	if (s->connection.data.virtual.ops->init)
+		return s->connection.data.virtual.ops->init(s);
+
+	return TRUE;
+}
+
 static gboolean connect_server(struct irc_network *s)
 {
 	struct network_config *nc = s->private_data;
@@ -820,23 +855,7 @@ static gboolean connect_server(struct irc_network *s)
 		return connect_program(s);
 
 	case NETWORK_VIRTUAL:
-		s->connection.data.virtual.ops = g_hash_table_lookup(
-								virtual_network_ops, 
-								nc->type_settings.virtual_type);
-		if (!s->connection.data.virtual.ops) 
-			return FALSE;
-
-		s->state = network_state_init(nc->nick, nc->username, 
-									  get_my_hostname());
-		s->state->userdata = s;
-		s->state->log = state_log_helper;
-		s->linestack = new_linestack(s);
-		s->connection.state = NETWORK_CONNECTION_STATE_MOTD_RECVD;
-
-		if (s->connection.data.virtual.ops->init)
-			return s->connection.data.virtual.ops->init(s);
-
-		return TRUE;
+		return connect_virtual(s);
 	default: g_assert_not_reached();
 	}
 
@@ -860,7 +879,7 @@ struct irc_network *irc_network_new(const struct irc_network_callbacks *callback
 	s->private_data = private_data;
 	s->reconnect_interval = ((struct network_config *)private_data)->reconnect_interval == -1?DEFAULT_RECONNECT_INTERVAL:((struct network_config *)private_data)->reconnect_interval;
 	s->info = network_info_init();
-	s->info->name = g_strdup(((struct network_config *)private_data)->name);
+	s->name = g_strdup(((struct network_config *)private_data)->name);
 	s->info->ircd = g_strdup("ctrlproxy");
 	s->info->forced_nick_changes = TRUE; /* Forced nick changes are done by ctrlproxy */
 
@@ -900,6 +919,7 @@ static void free_network(struct irc_network *s)
 	ssl_free_client_credentials(s->ssl_credentials);
 #endif
 
+	g_free(s->name);
 	g_free(s);
 }
 
@@ -969,7 +989,7 @@ struct irc_network *find_network(GList *networks, const char *name)
 	GList *gl;
 	for (gl = networks; gl; gl = gl->next) {
 		struct irc_network *n = gl->data;
-		if (n->info->name && !g_strcasecmp(n->info->name, name)) 
+		if (n->name && !g_strcasecmp(n->name, name)) 
 			return n;
 	}
 
