@@ -177,6 +177,12 @@ char *network_info_string(struct irc_network_info *info)
 	if (info->accept)
 		fs = g_list_append(fs, g_strdup("ACCEPT"));
 
+	if (info->esilence)
+		fs = g_list_append(fs, g_strdup("ESILENCE"));
+
+	if (info->uhnames)
+		fs = g_list_append(fs, g_strdup("UHNAMES"));
+
 	if (info->keylen != 0)
 		fs = g_list_append(fs, g_strdup_printf("KEYLEN=%d", info->keylen));
 
@@ -264,6 +270,31 @@ char *network_info_string(struct irc_network_info *info)
 	return ret;
 }
 
+static void check_chanmodes_inconsistency(struct irc_network_info *info)
+{
+	int i, j;
+	if (info->supported_channel_modes == NULL)
+		return;
+
+	for (i = 0; info->chanmodes[i]; i++) {
+		for (j = 0; info->chanmodes[i][j]; j++) {
+			if (!strchr(info->supported_channel_modes, info->chanmodes[i][j])) {
+				char *new_modes;
+				if (info->name != NULL) 
+					log_global(LOG_WARNING, 
+						   "Server for %s reported inconsistent supported modes. %c was in CHANMODES but not in 004 line.", info->name, info->chanmodes[i][j]);
+				else
+					log_global(LOG_WARNING, 
+						   "Server reported inconsistent supported modes. %c was in CHANMODES but not in 004 line.", info->chanmodes[i][j]);
+				new_modes = g_strdup_printf("%s%c", info->supported_channel_modes, info->chanmodes[i][j]);
+				g_free(info->supported_channel_modes);
+				info->supported_channel_modes = new_modes;
+			}
+		}
+	}
+}
+
+
 void network_info_parse(struct irc_network_info *info, const char *parameter)
 {
 	char *sep;
@@ -364,6 +395,10 @@ void network_info_parse(struct irc_network_info *info, const char *parameter)
 		info->callerid = TRUE;
 	} else if (!g_strcasecmp(key, "ACCEPT")) {
 		info->accept = TRUE;
+	} else if (!g_strcasecmp(key, "ESILENCE")) {
+		info->esilence = TRUE;
+	} else if (!g_strcasecmp(key, "UHNAMES")) {
+		info->uhnames = TRUE;
 	} else if (!g_strcasecmp(key, "KEYLEN")) {
 		info->keylen = atoi(val);
 	} else if (!g_strcasecmp(key, "SILENCE")) {
@@ -377,8 +412,19 @@ void network_info_parse(struct irc_network_info *info, const char *parameter)
 		info->chantypes = g_strdup(val);
 	} else if (!g_strcasecmp(key, "CHANMODES")) {
 		g_strfreev(info->chanmodes);
+		info->chanmodes_a = info->chanmodes_b = 
+			info->chanmodes_c = info->chanmodes_d = NULL;
 		info->chanmodes = g_strsplit(val, ",", 4);
-		/* FIXME: Make sure info->chanmodes length is exactly 4 */
+		if (g_strv_length(info->chanmodes) != 4) {
+			log_global(LOG_WARNING, "Expected 4-tuple for CHANMODES, received tuple was of size %d", g_strv_length(info->chanmodes));
+		} else {
+			info->chanmodes_a = info->chanmodes[0];
+			info->chanmodes_b = info->chanmodes[1];
+			info->chanmodes_c = info->chanmodes[2];
+			info->chanmodes_d = info->chanmodes[2];
+
+			check_chanmodes_inconsistency(info);
+		}
 	} else if (!g_strcasecmp(key, "CHANLIMIT")) {
 		g_free(info->chanlimit);
 		info->chanlimit = g_strdup(val);
@@ -613,14 +659,15 @@ const static char *default_chanmodes[] = {
 	"beI", "k" , "l" , "imnpsta" 
 };
 
-int network_chanmode_type(char m, struct irc_network_info *n)
+enum chanmode_type network_chanmode_type(char m, struct irc_network_info *n)
 {
 	int i;
 	for (i = 0; i < 4; i++) {
 		if (strchr((n && n->chanmodes)?n->chanmodes[i]:default_chanmodes[i], m)) {
-			return i;
+			return i+1;
 		}
 	}
 
-	return 3;
+	return CHANMODE_UNKNOWN;
 }
+
