@@ -35,7 +35,12 @@
 #include <netdb.h>
 #include "socks.h"
 
+#ifdef HAVE_GSSAPI
+static gboolean gssapi_fail(struct pending_client *pc);
+#endif
+
 struct listener_iochannel {
+	struct irc_listener *listener;
 	char address[NI_MAXHOST];
 	char port[NI_MAXSERV];
 	gint watch_id;
@@ -48,10 +53,57 @@ static gboolean handle_client_socks_data(GIOChannel *ioc,
 
 static gboolean kill_pending_client(struct pending_client *pc)
 {
+#ifdef HAVE_GSSAPI
+	guint32 major_status, minor_status;
+#endif
+
 	pc->listener->pending = g_list_remove(pc->listener->pending, pc);
 
 	g_source_remove(pc->watch_id);
 
+#ifdef HAVE_GSSAPI
+	if (pc->authn_name != GSS_C_NO_NAME) {
+		major_status = gss_release_name(&minor_status, &pc->authn_name);
+
+		if (GSS_ERROR(major_status)) {
+			log_gssapi(pc->listener, LOG_WARNING, 
+					   "releasing name", major_status, minor_status);
+			gssapi_fail(pc);
+		}
+	}
+
+	if (pc->gss_ctx != GSS_C_NO_CONTEXT) {
+		major_status = gss_delete_sec_context(&minor_status,
+						      &pc->gss_ctx,
+						      GSS_C_NO_BUFFER);
+		if (GSS_ERROR(major_status)) {
+			log_gssapi(pc->listener, LOG_WARNING, 
+					   "deleting context name", major_status, minor_status);
+			gssapi_fail(pc);
+		}
+	}
+
+	if (pc->service_cred != GSS_C_NO_CREDENTIAL) {
+		major_status = gss_release_cred(&minor_status, &pc->service_cred);
+		if (GSS_ERROR(major_status)) {
+			log_gssapi(pc->listener, LOG_WARNING, 
+					   "releasing credentials", major_status, minor_status);
+			return FALSE;
+		}
+	}
+
+	if (pc->gss_service != GSS_C_NO_NAME) {
+		guint32 major_status, minor_status;
+		major_status = gss_release_name(&minor_status, &pc->gss_service);
+
+		if (GSS_ERROR(major_status)) {
+			log_gssapi(pc->listener, LOG_WARNING, 
+					   "releasing name", major_status, minor_status);
+			return FALSE;
+		}
+	}
+#endif
+
 	g_free(pc->clientname);
 
 	g_free(pc);
@@ -59,6 +111,33 @@ static gboolean kill_pending_client(struct pending_client *pc)
 	return TRUE;
 }
 
+#ifdef HAVE_GSSAPI
+void log_gssapi(struct irc_listener *l, enum log_level level, const char *message, guint32 major_status, guint32 minor_status)
+{
+	guint32 err_major_status, err_minor_status;
+	guint32	msg_ctx = 0;
+	gss_buffer_desc major_status_string = GSS_C_EMPTY_BUFFER, 
+					minor_status_string = GSS_C_EMPTY_BUFFER;
+
+	err_major_status = gss_display_status( &err_minor_status, major_status, 
+										   GSS_C_GSS_CODE, GSS_C_NO_OID, 
+										   &msg_ctx, &major_status_string );
+
+	if( !GSS_ERROR( err_major_status ) )
+    	err_major_status = gss_display_status( &err_minor_status, minor_status,
+											   GSS_C_MECH_CODE, GSS_C_NULL_OID,
+											   &msg_ctx, &minor_status_string );
+
+	listener_log( level, l, "GSSAPI: %s: %s, %s", message, 
+		      (char *)major_status_string.value, 
+		      (char *)minor_status_string.value);
+
+	gss_release_buffer( &err_minor_status, &major_status_string );
+	gss_release_buffer( &err_minor_status, &minor_status_string );
+}
+#endif
+
+
 gboolean listener_stop(struct irc_listener *l)
 {
 	while (l->incoming != NULL) {
@@ -66,8 +145,10 @@ gboolean listener_stop(struct irc_listener *l)
 
 		g_source_remove(lio->watch_id);
 		
-		listener_log(LOG_INFO, l, "Stopped listening at %s:%s", lio->address, 
-					 lio->port);
+		if (strcmp(lio->address, "") != 0)
+			listener_log(LOG_INFO, l, "Stopped listening at %s:%s", lio->address, 
+						 lio->port);
+
 		g_free(lio);
 
 		l->incoming = g_list_remove(l->incoming, lio);
@@ -213,6 +294,12 @@ struct pending_client *listener_new_pending_client(struct irc_listener *listener
 	g_io_channel_set_encoding(c, NULL, NULL);
 	g_io_channel_set_flags(c, G_IO_FLAG_NONBLOCK, NULL);
 	pc = g_new0(struct pending_client, 1);
+#ifdef HAVE_GSSAPI
+	pc->authn_name = GSS_C_NO_NAME;
+	pc->gss_ctx = GSS_C_NO_CONTEXT;
+	pc->gss_service = GSS_C_NO_NAME;
+	pc->service_cred = GSS_C_NO_CREDENTIAL;
+#endif
 	pc->connection = c;
 	pc->watch_id = g_io_add_watch(c, G_IO_IN | G_IO_HUP, handle_client_receive, pc);
 	pc->listener = listener;
@@ -245,19 +332,47 @@ static gboolean handle_new_client(GIOChannel *c_server, GIOCondition condition,
 	return TRUE;
 }
 
+void listener_add_iochannel(struct irc_listener *l, GIOChannel *ioc,
+							const char *address, const char *port)
+{
+	struct listener_iochannel *lio;
+
+	lio = g_new0(struct listener_iochannel, 1);
+	lio->listener = l;
+
+	g_io_channel_set_close_on_unref(ioc, TRUE);
+	g_io_channel_set_encoding(ioc, NULL, NULL);
+	lio->watch_id = g_io_add_watch(ioc, G_IO_IN, handle_new_client, l);
+	g_io_channel_unref(ioc);
+	l->incoming = g_list_append(l->incoming, lio);
+
+	if (address == NULL)
+		strcpy(lio->address, "");
+	else
+		strncpy(lio->address, address, NI_MAXHOST);
+	if (port == NULL)
+		strcpy(lio->address, "");
+	else 
+		strncpy(lio->port, port, NI_MAXSERV);
+
+	if (address != NULL)
+		listener_log(LOG_INFO, l, "Listening on %s:%s", 
+					 address, port);
+	l->active = TRUE;
+}
+
 /**
  * Start a listener.
  *
  * @param l Listener to start.
  */
-gboolean listener_start(struct irc_listener *l, const char *address, const char *port)
+gboolean listener_start_tcp(struct irc_listener *l, const char *address, const char *port)
 {
 	int sock = -1;
 	const int on = 1;
 	struct addrinfo *res, *all_res;
 	int error;
 	struct addrinfo hints;
-	struct listener_iochannel *lio;
 
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_family = PF_UNSPEC;
@@ -275,20 +390,23 @@ gboolean listener_start(struct irc_listener *l, const char *address, const char
 
 	error = getaddrinfo(address, port, &hints, &all_res);
 	if (error) {
-		listener_log(LOG_ERROR, l, "Can't get address for %s:%s", address?address:"", port);
+		listener_log(LOG_ERROR, l, "Can't get address for %s:%s: %s", 
+					 address?address:"", port, gai_strerror(error));
 		return FALSE;
 	}
 
 	for (res = all_res; res; res = res->ai_next) {
 		GIOChannel *ioc;
+		char canon_address[NI_MAXHOST], canon_port[NI_MAXSERV];
 
-		lio = g_new0(struct listener_iochannel, 1);
-
-		if (getnameinfo(res->ai_addr, res->ai_addrlen, 
-						lio->address, NI_MAXHOST, lio->port, NI_MAXSERV, 
-						NI_NUMERICHOST | NI_NUMERICSERV) != 0) {
-			strcpy(lio->address, "");
-			strcpy(lio->port, "");
+		error = getnameinfo(res->ai_addr, res->ai_addrlen, 
+							canon_address, NI_MAXHOST, 
+							canon_port, NI_MAXSERV, NI_NUMERICHOST|NI_NUMERICSERV);
+		if (error < 0) {
+			listener_log(LOG_WARNING, l, "error looking up canonical name: %s", 
+						 gai_strerror(error));
+			strcpy(canon_address, "");
+			strcpy(canon_port, "");
 		}
 
 		sock = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
@@ -296,7 +414,6 @@ gboolean listener_start(struct irc_listener *l, const char *address, const char
 			listener_log(LOG_ERROR, l, "error creating socket: %s", 
 						 strerror(errno));
 			close(sock);
-			g_free(lio);
 			continue;
 		}
 
@@ -311,7 +428,6 @@ gboolean listener_start(struct irc_listener *l, const char *address, const char
 						   strerror(errno));
 			}
 			close(sock);
-			g_free(lio);
 			continue;
 		}
 
@@ -319,7 +435,6 @@ gboolean listener_start(struct irc_listener *l, const char *address, const char
 			listener_log(LOG_ERROR, l, "error listening on socket: %s", 
 						strerror(errno));
 			close(sock);
-			g_free(lio);
 			continue;
 		}
 
@@ -329,20 +444,10 @@ gboolean listener_start(struct irc_listener *l, const char *address, const char
 			listener_log(LOG_ERROR, l,
 						"Unable to create GIOChannel for server socket");
 			close(sock);
-			g_free(lio);
 			continue;
 		}
 
-		g_io_channel_set_close_on_unref(ioc, TRUE);
-
-		g_io_channel_set_encoding(ioc, NULL, NULL);
-		lio->watch_id = g_io_add_watch(ioc, G_IO_IN, handle_new_client, l);
-		g_io_channel_unref(ioc);
-		l->incoming = g_list_append(l->incoming, lio);
-
-		listener_log(LOG_INFO, l, "Listening on %s:%s", 
-						lio->address, lio->port);
-		l->active = TRUE;
+		listener_add_iochannel(l, ioc, canon_address, canon_port);
 	}
 
 	freeaddrinfo(all_res);
@@ -370,7 +475,8 @@ gboolean listener_socks_reply(struct pending_client *pc, guint8 err, guint8 atyp
 	memcpy(header+4, data, data_len);
 	*((guint16 *)(header+4+data_len)) = htons(port);
 
-	status = g_io_channel_write_chars(pc->connection, header, 6 + data_len, &read, NULL);
+	status = g_io_channel_write_chars(pc->connection, header, 
+									  6 + data_len, &read, NULL);
 
 	g_free(header);
 
@@ -395,10 +501,36 @@ static gboolean pass_acceptable(struct pending_client *cl)
 	return TRUE; /* FIXME: Check whether there is a password specified */
 }
 
+static gboolean on_socks_pass_accepted(struct pending_client *cl, gboolean accepted)
+{
+	gchar header[2];
+	GIOStatus status;
+	gsize read;
+
+	header[0] = 0x1;
+	/* set to non-zero if invalid */
+	header[1] = accepted?0x0:0x1;
+
+	status = g_io_channel_write_chars(cl->connection, header, 2, &read, NULL);
+	if (status != G_IO_STATUS_NORMAL) {
+		return FALSE;
+	} 
+
+	g_io_channel_flush(cl->connection, NULL);
+
+	if (header[1] == 0x0) {
+		cl->socks.state = SOCKS_STATE_NORMAL;		
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
 static gboolean pass_handle_data(struct pending_client *cl)
 {
 	gchar header[2];
 	gsize read;
+	gboolean accepted;
 	GIOStatus status;
 	gchar uname[0x100], pass[0x100];
 
@@ -431,27 +563,195 @@ static gboolean pass_handle_data(struct pending_client *cl)
 
 	pass[(guint8)header[0]] = '\0';
 
-	header[0] = 0x1;
+	accepted = cl->listener->ops->socks_auth_simple(cl, uname, pass, on_socks_pass_accepted);
+	return TRUE;
+}
 
-	/* set to non-zero if invalid */
-	header[1] = cl->listener->ops->socks_auth_simple(cl, uname, pass)?0x0:0x1;
+#ifdef HAVE_GSSAPI
+static gboolean gssapi_acceptable (struct pending_client *pc)
+{
+	struct irc_listener *l = pc->listener;
+	guint32 major_status, minor_status;
+	gss_buffer_desc inbuf;
+	char *principal_name;
+	int fd, error;
+	char address[NI_MAXHOST];
+	struct sockaddr_storage sockaddr;
+	socklen_t sockaddr_len = sizeof(sockaddr);
 
-	status = g_io_channel_write_chars(cl->connection, header, 2, &read, NULL);
+	fd = g_io_channel_unix_get_fd(pc->connection);
+
+	error = getsockname(fd, (struct sockaddr *)&sockaddr, &sockaddr_len);
+	if (error < 0) {
+		listener_log(LOG_WARNING, pc->listener, 
+					 "Unable to get own socket address");
+		return FALSE;
+	}
+
+	error = getnameinfo((struct sockaddr *)&sockaddr, sockaddr_len, 
+						address, NI_MAXHOST, 
+						NULL, 0, NI_NAMEREQD);
+	if (error < 0) {
+		listener_log(LOG_WARNING, l, "error looking up canonical name: %s", 
+						 gai_strerror(error));
+		return FALSE;
+	}
+	
+	principal_name = g_strdup_printf("socks@%s", address);
+
+	inbuf.length = strlen(principal_name);
+	inbuf.value = principal_name;
+
+	major_status = gss_import_name(&minor_status, &inbuf, 
+					   GSS_C_NT_HOSTBASED_SERVICE,
+					   &pc->gss_service);
+
+	g_free(principal_name);
+
+	if (GSS_ERROR(major_status)) {
+		log_gssapi(l, LOG_ERROR, "importing principal name",
+				   major_status, minor_status);
+		gssapi_fail(pc);
+		return FALSE;
+	}
+
+	major_status = gss_acquire_cred(&minor_status, pc->gss_service, 0, 
+					GSS_C_NULL_OID_SET, GSS_C_ACCEPT,
+					&pc->service_cred, NULL, NULL);
+
+	if (GSS_ERROR(major_status)) {
+		log_gssapi(l, LOG_ERROR, "acquiring service credentials",
+				   major_status, minor_status);
+		gssapi_fail(pc);
+
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean gssapi_fail(struct pending_client *pc)
+{
+	char header[2];
+	GIOStatus status;
+	gsize read;
+
+	header[0] = 1; /* SOCKS_GSSAPI_VERSION */
+	header[1] = 0xff; /* Message abort */
+
+	status = g_io_channel_write_chars(pc->connection, header, 2, &read, NULL);
 	if (status != G_IO_STATUS_NORMAL) {
 		return FALSE;
 	} 
 
-	g_io_channel_flush(cl->connection, NULL);
+	g_io_channel_flush(pc->connection, NULL);
 
-	if (header[1] == 0x0) {
-		cl->socks.state = SOCKS_STATE_NORMAL;		
-		return TRUE;
-	} else {
-		listener_log(LOG_WARNING, cl->listener, "Password mismatch for user %s", uname);
+	return FALSE;
+}
+
+static gboolean gssapi_handle_data (struct pending_client *pc)
+{
+	guint32 major_status, minor_status;
+	gss_buffer_desc outbuf;
+	gss_buffer_desc inbuf;
+	gchar header[4];
+	gchar *packet;
+	gsize read;
+	GIOStatus status;
+	guint16 len;
+
+	/*
+    + ver  | mtyp | len  |       token           |
+    + 0x01 | 0x01 | 0x02 | up to 2^16 - 1 octets |
+	*/
+
+	status = g_io_channel_read_chars(pc->connection, header, 4, &read, NULL);
+	if (status != G_IO_STATUS_NORMAL) {
 		return FALSE;
 	}
+
+	if (header[0] != 1) {
+		listener_log(LOG_WARNING, pc->listener, "Invalid version %d for SOCKS/GSSAPI protocol", header[0]);
+		return FALSE;
+	}
+
+	if (header[1] != 1) {
+		listener_log(LOG_WARNING, pc->listener, "Invalid SOCKS/GSSAPI command %d", header[1]);
+		return FALSE;
+	}
+
+	len = ntohs(*((uint16_t *)(header+2)));
+
+	inbuf.value = g_malloc(len);
+	status = g_io_channel_read_chars(pc->connection, inbuf.value, len, &inbuf.length, NULL);
+	if (status != G_IO_STATUS_NORMAL) {
+		return FALSE;
+	}
+
+	major_status = gss_accept_sec_context (
+		&minor_status,
+		&pc->gss_ctx,
+		pc->service_cred,
+		&inbuf,
+		GSS_C_NO_CHANNEL_BINDINGS,
+		&pc->authn_name, 
+		NULL, /* mech_type */
+		&outbuf,
+		NULL, /* ret_flags */
+		NULL, /* time_rec */
+		NULL  /* delegated_cred_handle */
+	);
+
+	g_free(inbuf.value);
+	
+	if (GSS_ERROR(major_status)) {
+		log_gssapi(pc->listener, LOG_WARNING, "processing incoming data", 
+				   major_status, minor_status);
+		gssapi_fail(pc);
+		return FALSE;
+	} 
+
+	packet = g_malloc(4+outbuf.length);
+	packet[0] = SOCKS_VERSION; /* SOCKS_GSSAPI_VERSION */
+	packet[1] = 1; /* Authorization message */
+	*((uint16_t *)(packet+2)) = htons(outbuf.length);
+	
+	memcpy(packet+4, outbuf.value, outbuf.length);
+
+	status = g_io_channel_write_chars(pc->connection, packet, 4+outbuf.length, &read, NULL);
+	g_free(packet);
+
+	if (status != G_IO_STATUS_NORMAL) {
+		gssapi_fail(pc);
+		return FALSE;
+	} 
+
+	major_status = gss_release_buffer(&minor_status, &outbuf);
+	if (GSS_ERROR(major_status)) {
+		log_gssapi(pc->listener, LOG_WARNING, "processing incoming data", 
+				   major_status, minor_status);
+		gssapi_fail(pc);
+		return FALSE;
+	} 
+
+	g_io_channel_flush(pc->connection, NULL);
+
+	g_assert(major_status == GSS_S_COMPLETE);
+
+	if (!pc->listener->ops->socks_gssapi(pc, pc->authn_name)) {
+		gssapi_fail(pc);
+		return FALSE;
+	}
+
+	if (major_status == GSS_S_CONTINUE_NEEDED) {
+		return TRUE;
+	}
+
+	return TRUE;
 }
 
+#endif
+
 /**
  * Socks methods.
  */
@@ -462,7 +762,9 @@ static struct socks_method {
 	gboolean (*handle_data) (struct pending_client *cl);
 } socks_methods[] = {
 	{ SOCKS_METHOD_NOAUTH, "none", anon_acceptable, NULL },
-	{ SOCKS_METHOD_GSSAPI, "gssapi", NULL, NULL },
+#ifdef HAVE_GSSAPI
+	{ SOCKS_METHOD_GSSAPI, "gssapi", gssapi_acceptable, gssapi_handle_data },
+#endif
 	{ SOCKS_METHOD_USERNAME_PW, "username/password", pass_acceptable, pass_handle_data },
 	{ -1, NULL, NULL }
 };
