@@ -130,83 +130,61 @@ static void free_channel_nick(struct channel_nick *n)
 	g_free(n);
 }
 
-static void free_invitelist(struct irc_channel_state *c)
+gboolean nicklist_add_entry(GList **nicklist, const char *opt_arg,
+								   const char *by_nick, time_t at)
 {
-	GList *g;
-	g_assert(c);
+	struct nicklist_entry *be;
+	be = g_new0(struct nicklist_entry, 1);
+	be->time_set = at;
+	be->hostmask = g_strdup(opt_arg);
+	be->by = (by_nick?g_strdup(by_nick):NULL);
+	*nicklist = g_list_append(*nicklist, be);
 
-	g = c->invitelist;
-	while(g) {
-		g_free(g->data);
-		g = g_list_remove(g, g->data);
-	}
-	c->invitelist = NULL;
+	return TRUE;
 }
 
-
-static void free_exceptlist(struct irc_channel_state *c)
+gboolean nicklist_remove_entry(GList **nicklist, const char *hostmask)
 {
-	GList *g;
-	g_assert(c);
+	struct nicklist_entry *be;
 
-	g = c->exceptlist;
-	while(g) {
-		g_free(g->data);
-		g = g_list_remove(g, g->data);
-	}
-	c->exceptlist = NULL;
+	be = find_nicklist_entry(*nicklist, hostmask);
+	if (be == NULL)
+		return FALSE;
+	*nicklist = g_list_remove(*nicklist, be);
+	free_nicklist_entry(be);
+	return TRUE;
 }
 
-static void free_banlist_entry(struct banlist_entry *be)
+void free_nicklist_entry(struct nicklist_entry *be)
 {
 	g_free(be->hostmask);
 	g_free(be->by);
 	g_free(be);
 }
 
-static char *find_exceptlist_entry(GList *entries, const char *entry)
+struct nicklist_entry *find_nicklist_entry(GList *entries, const char *hostmask)
 {
 	GList *gl;
 	for (gl = entries; gl; gl = gl->next) {
-		if (!strcmp(gl->data, entry))
-			return gl->data;
-	}
-	return NULL;
-}
-
-static char *find_realnamebanlist_entry(GList *entries, const char *entry)
-{
-	GList *gl;
-	for (gl = entries; gl; gl = gl->next) {
-		if (!strcmp(gl->data, entry))
-			return gl->data;
-	}
-	return NULL;
-}
-
-static struct banlist_entry *find_banlist_entry(GList *entries, const char *hostmask)
-{
-	GList *gl;
-	for (gl = entries; gl; gl = gl->next) {
-		struct banlist_entry *be = gl->data;
+		struct nicklist_entry *be = gl->data;
 		if (!strcmp(be->hostmask, hostmask))
 			return be;
 	}
 	return NULL;
 }
 
-static void free_banlist(struct irc_channel_state *c)
+void free_nicklist(GList **nicklist)
 {
 	GList *g;
 	
-	g_assert(c);
-	g = c->banlist;
+	g = *nicklist;
 	while(g) {
-		struct banlist_entry *be = g->data;
+		struct nicklist_entry *be = g->data;
 		g = g_list_remove(g, be);
-		free_banlist_entry(be);
+		free_nicklist_entry(be);
 	}
-	c->banlist = NULL;
+
+	*nicklist = NULL;
 }
 
 static void free_names(struct irc_channel_state *c)
@@ -219,15 +197,19 @@ static void free_names(struct irc_channel_state *c)
 
 void free_channel_state(struct irc_channel_state *c)
 {
+	int i;
 	if (c == NULL)
 		return;
 	free_names(c);
 	g_free(c->name);
 	g_free(c->topic);
 	g_free(c->topic_set_by);
-	g_free(c->key);
 	g_assert(c->network);
 	c->network->channels = g_list_remove(c->network->channels, c);
+	for (i = 0; i < MAXMODES; i++)
+		g_free(c->chanmode_option[i]);
+	for (i = 0; i < MAXMODES; i++)
+		free_nicklist(&c->chanmode_nicklist[i]);
 	g_free(c);
 }
 
@@ -593,7 +575,7 @@ static void handle_333(struct irc_network_state *s, const struct irc_line *l)
 		return;
 	}
 
-	c->topic_set_time = strtoul(l->args[4], NULL, 0);
+	c->topic_set_time = strtol(l->args[4], NULL, 0);
 	c->topic_set_by = g_strdup(l->args[3]);
 }
 
@@ -651,6 +633,7 @@ static void handle_end_names(struct irc_network_state *s, const struct irc_line
 static void handle_invitelist_entry(struct irc_network_state *s, const struct irc_line *l) 
 {
 	struct irc_channel_state *c = find_channel(s, l->args[2]);
+	GList **list;
 	
 	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
@@ -659,27 +642,33 @@ static void handle_invitelist_entry(struct irc_network_state *s, const struct ir
 		return;
 	}
 
+	list = &channel_mode_nicklist(c, 'I');
+
 	if (!c->invitelist_started) {
-		free_invitelist(c);
+		free_nicklist(list);
 		c->invitelist_started = TRUE;
 	}
 
-	c->invitelist = g_list_append(c->invitelist, g_strdup(l->args[3]));
+	nicklist_add_entry(list, l->args[3], NULL, 0);
 }
 
 static void handle_end_invitelist(struct irc_network_state *s, const struct irc_line *l) 
 {
 	struct irc_channel_state *c = find_channel(s, l->args[2]);
-	if (c != NULL)
-		c->invitelist_started = FALSE;
-	else 
+	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
 			  "Can't end invitelist for %s: channel not found", l->args[2]);
+		return;
+	}
+
+	c->invitelist_started = FALSE;
+	channel_mode_nicklist_present(c,'I') = TRUE;
 }
 
 static void handle_exceptlist_entry(struct irc_network_state *s, const struct irc_line *l) 
 {
 	struct irc_channel_state *c = find_channel(s, l->args[2]);
+	GList **list;
 	
 	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
@@ -688,22 +677,26 @@ static void handle_exceptlist_entry(struct irc_network_state *s, const struct ir
 		return;
 	}
 
+	list = &channel_mode_nicklist(c, 'e');
+
 	if (!c->exceptlist_started) {
-		free_exceptlist(c);
+		free_nicklist(list);
 		c->exceptlist_started = TRUE;
 	}
 
-	c->exceptlist = g_list_append(c->exceptlist, g_strdup(l->args[3]));
+	nicklist_add_entry(list, l->args[3], NULL, 0);
 }
 
 static void handle_end_exceptlist(struct irc_network_state *s, const struct irc_line *l) 
 {
 	struct irc_channel_state *c = find_channel(s, l->args[2]);
-	if (c != NULL)
-		c->exceptlist_started = FALSE;
-	else 
+	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
 			"Can't end exceptlist for %s: channel not found", l->args[2]);
+		return;
+	}
+	c->exceptlist_started = FALSE;
+	channel_mode_nicklist_present(c, 'e') = TRUE;
 }
 
 
@@ -711,7 +704,7 @@ static void handle_end_exceptlist(struct irc_network_state *s, const struct irc_
 static void handle_banlist_entry(struct irc_network_state *s, const struct irc_line *l) 
 {
 	struct irc_channel_state *c = find_channel(s, l->args[2]);
-	struct banlist_entry *be;
+	GList **list;
 	
 	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
@@ -720,31 +713,29 @@ static void handle_banlist_entry(struct irc_network_state *s, const struct irc_l
 		return;
 	}
 
+	list = &channel_mode_nicklist(c, 'b');
+
 	if (!c->banlist_started) {
-		free_banlist(c);
+		free_nicklist(list);
 		c->banlist_started = TRUE;
 	}
 
-	be = g_new0(struct banlist_entry, 1);
-	be->hostmask = g_strdup(l->args[3]);
-	if (l->args[4] != NULL) {
-		be->by = g_strdup(l->args[4]);
-		if (l->args[5]) 
-			be->time_set = atol(l->args[5]);
-	}
-
-	c->banlist = g_list_append(c->banlist, be);
+	nicklist_add_entry(list, l->args[3], l->args[4], 
+					   (l->args[4] && l->args[5])?atol(l->args[5]):0);
 }
 
 static void handle_end_banlist(struct irc_network_state *s, const struct irc_line *l) 
 {
 	struct irc_channel_state *c = find_channel(s, l->args[2]);
 
-	if (c != NULL)
-		c->banlist_started = FALSE;
-	else 
+	if (c == NULL) {
 		network_state_log(LOG_WARNING, s, 
 				"Can't end banlist for %s: channel not found", l->args[2]);
+		return;
+	}
+
+	c->banlist_started = FALSE;
+	channel_mode_nicklist_present(c,'b') = TRUE;
 }
 
 static void handle_whoreply(struct irc_network_state *s, const struct irc_line *l) 
@@ -841,122 +832,43 @@ static int channel_state_change_mode(struct irc_network_state *s, struct network
 {
 	struct irc_network_info *info = s->info;
 
-	if (!is_channel_mode(info, mode)) {
-		network_state_log(LOG_WARNING, s, "Mode '%c' set on channel %s is not in the supported list of channel modes from the server", mode, c->name);
-	}
-
-	if (mode == 'b') { /* Ban */
-		struct banlist_entry *be;
+	enum chanmode_type cmt = network_chanmode_type(mode, s->info);
 
+	if (cmt == CHANMODE_NICKLIST) {
 		if (opt_arg == NULL) {
-			network_state_log(LOG_WARNING, s, "Missing argument for ban MODE set/unset");
+			network_state_log(LOG_WARNING, s, "Missing argument for %c MODE set/unset", mode);
 			return -1;
 		}
 
-		if (set) {
-			be = g_new0(struct banlist_entry, 1);
-			be->time_set = time(NULL);
-			be->hostmask = g_strdup(opt_arg);
-			be->by = (by?g_strdup(by->nick):NULL);
-			c->banlist = g_list_append(c->banlist, be);
-		} else {
-			be = find_banlist_entry(c->banlist, opt_arg);
-			if (be == NULL) {
-				network_state_log(LOG_WARNING, s, "Unable to remove nonpresent banlist entry '%s'", opt_arg);
-				return 1;
+		if (channel_mode_nicklist_present(c, mode)) {
+			if (set) {
+				nicklist_add_entry(&channel_mode_nicklist(c, mode), opt_arg, 
+								   by?by->nick:NULL, time(NULL));
+			} else {
+				if (!nicklist_remove_entry(&channel_mode_nicklist(c, mode), opt_arg))  {
+					network_state_log(LOG_WARNING, s, "Unable to remove nonpresent %c MODE entry '%s' on %s", mode, opt_arg, c->name);
+					return 1;
+				}
 			}
-			c->banlist = g_list_remove(c->banlist, be);
-			free_banlist_entry(be);
 		}
 		return 1;
-	} else if (mode == 'e') { /* Ban exemption */ 
-		if (opt_arg == NULL) {
-			network_state_log(LOG_WARNING, s, "Missing argument for ban exception MODE set/unset");
-			return -1;
-		}
-
-		if (set) {
-			c->exceptlist = g_list_append(c->exceptlist, g_strdup(opt_arg));
-		} else {
-			char *be = find_exceptlist_entry(c->exceptlist, opt_arg);
-			if (be == NULL) {
-				network_state_log(LOG_WARNING, s, "Unable to remove nonpresent ban except list entry '%s'", opt_arg);
-				return 1;
-			}
-			c->exceptlist = g_list_remove(c->exceptlist, be);
-			g_free(be);
-		}
-		return 1;
-	} else if (mode == 'J') { /* join throttling */
-		if (set) {
-			char **join_args;
-			if (opt_arg == NULL) {
-				network_state_log(LOG_WARNING, s, "Missing argument for realname ban MODE set/unset");
-				return -1;
-			}
-
-			join_args = g_strsplit(opt_arg, ",", 2);
-
-			if (join_args == NULL) {
-				network_state_log(LOG_ERROR, s, "Unable to split argument %s", opt_arg);
-				return -1;
-			}
-
-			if (join_args[0]) 
-				c->throttle_x = atoi(join_args[0]);
-			if (join_args[0] && join_args[1]) 
-				c->throttle_y = atoi(join_args[1]);
-		} else {
-			c->throttle_x = 0;
-			c->throttle_y = 0;
-		}
-
-		return 1;
-	} else if (mode == 'd') { /* Realname ban */
-		if (opt_arg == NULL) {
-			network_state_log(LOG_WARNING, s, "Missing argument for realname ban MODE set/unset");
-			return -1;
-		}
-
-		if (set) {
-			c->realnamebanlist = g_list_append(c->realnamebanlist, g_strdup(opt_arg));
-		} else {
-			char *be = find_realnamebanlist_entry(c->realnamebanlist, opt_arg);
-			if (be == NULL) {
-				network_state_log(LOG_WARNING, s, "Unable to remove nonpresent realname ban list entry '%s'", opt_arg);
-				return 1;
-			}
-			c->realnamebanlist = g_list_remove(c->realnamebanlist, be);
-			g_free(be);
-		}
-		return 1;	
-	} else if (mode == 'l') { /* Limit */
-		modes_change_mode(c->modes, set, 'l');
-		if (set) {
-			if (!opt_arg) {
-				network_state_log(LOG_WARNING, s, "Mode +l requires argument, but no argument found");
-				return -1;
-			}
-			c->limit = atol(opt_arg);
-		} else {
-			c->limit = 0;
-		}
-		return 1;
-	} else if (mode == 'k') {
-		modes_change_mode(c->modes, set, 'k');
+	} else if (cmt == CHANMODE_OPT_SETTING || cmt == CHANMODE_SETTING) {
 		if (set) {
 			if (opt_arg == NULL) {
-				network_state_log(LOG_WARNING, s, "Mode k requires argument, but no argument found");
+				network_state_log(LOG_WARNING, s, "Missing argument for MODE %c set", mode);
 				return -1;
 			}
 
-			g_free(c->key);
-			c->key = g_strdup(opt_arg);
+			g_free(channel_mode_option(c, mode));
+			channel_mode_option(c, mode) = g_strdup(opt_arg);
+
+			return 1;
 		} else {
-			g_free(c->key);
-			c->key = NULL;
+			g_free(channel_mode_option(c, mode));
+			channel_mode_option(c, mode) = NULL;
+
+			return 0;
 		}
-		return 1;
 	} else if (is_prefix_mode(info, mode)) {
 		struct channel_nick *n;
 		
@@ -976,9 +888,12 @@ static int channel_state_change_mode(struct irc_network_state *s, struct network
 			modes_unset_mode(n->modes, mode);
 		} 
 		return 1;
-	} else {
+	} else if (cmt == CHANMODE_BOOL) {
 		modes_change_mode(c->modes, set, mode);
 		return 0;
+	} else {
+		network_state_log(LOG_WARNING, s, "Mode '%c' set on channel %s is not in the supported list of channel modes from the server", mode, c->name);
+		return 1;
 	}
 }
 
@@ -1019,7 +934,9 @@ static void handle_mode(struct irc_network_state *s, const struct irc_line *l)
 													  l->args[arg]);
 					  if (ret == -1)
 						  return;
-					  arg += ret;
+					  g_assert(!(ret > 1));
+					  if (l->args[arg] != NULL)
+						  arg += ret;
 					  break;
 			}
 		}
