@@ -46,7 +46,7 @@ static void list_networks_helper(admin_handle h);
  */
 static char *admin_hostmask(struct irc_network *n)
 {
-	return g_strdup_printf(ADMIN_NICK"!"ADMIN_USERNAME"@%s", n->info->name);
+	return g_strdup_printf(ADMIN_NICK"!"ADMIN_USERNAME"@%s", n->name);
 }
 
 /**
@@ -62,8 +62,8 @@ static void privmsg_admin_out(admin_handle h, const char *data)
 	char *hostmask;
 
 	hostmask = admin_hostmask(c->network);
-	if (c->network->state != NULL) 
-		nick = c->network->state->me.nick;
+	if (c->network->external_state != NULL) 
+		nick = c->network->external_state->me.nick;
 	client_send_args_ex(c, hostmask, "NOTICE", nick, data, NULL);
 
 	g_free(hostmask);
@@ -93,7 +93,7 @@ static void network_admin_out(admin_handle h, const char *data)
  * @param args Arguments
  * @param userdata User context data (ignored)
  */
-static void cmd_help(admin_handle h, char **args, void *userdata)
+static void cmd_help(admin_handle h, const char * const *args, void *userdata)
 {
 	const char *s;
 
@@ -167,7 +167,7 @@ void admin_out(admin_handle h, const char *fmt, ...)
  * @param args String arguments, argv-style.
  * @param userdata Optional user data, always NULL.
  */
-static void cmd_add_network (admin_handle h, char **args, void *userdata)
+static void cmd_add_network (admin_handle h, const char * const *args, void *userdata)
 {
 	if (args[1] == NULL) {
 		admin_out(h, "No name specified");
@@ -177,7 +177,7 @@ static void cmd_add_network (admin_handle h, char **args, void *userdata)
 	add_network_helper(h, args[1]);
 }
 
-static void cmd_del_network (admin_handle h, char **args, void *userdata)
+static void cmd_del_network (admin_handle h, const char * const *args, void *userdata)
 {
 	if (args[1] == NULL) {
 		admin_out(h, "Not enough parameters");
@@ -187,7 +187,7 @@ static void cmd_del_network (admin_handle h, char **args, void *userdata)
 	del_network_helper(h, args[1]);
 }
 
-static void cmd_add_server (admin_handle h, char **args, void *userdata)
+static void cmd_add_server (admin_handle h, const char * const *args, void *userdata)
 {
 	struct irc_network *n;
 	struct tcp_server_config *s;
@@ -230,7 +230,7 @@ static void cmd_add_server (admin_handle h, char **args, void *userdata)
 	admin_out(h, "Server added to `%s'", args[1]);
 }
 
-static void cmd_connect_network (admin_handle h, char **args, void *userdata)
+static void cmd_connect_network (admin_handle h, const char * const *args, void *userdata)
 {
 	struct irc_network *s;
 	if (!args[1]) {
@@ -267,7 +267,7 @@ static void cmd_connect_network (admin_handle h, char **args, void *userdata)
 	}
 }
 
-static void cmd_disconnect_network (admin_handle h, char **args, void *userdata)
+static void cmd_disconnect_network (admin_handle h, const char * const *args, void *userdata)
 {
 	struct irc_network *n;
 	struct network_config *nc;
@@ -289,18 +289,18 @@ static void cmd_disconnect_network (admin_handle h, char **args, void *userdata)
 	nc = n->private_data;
 
 	if (n->connection.state == NETWORK_CONNECTION_STATE_NOT_CONNECTED) {
-		admin_out(h, "Already disconnected from `%s'", n->info->name);
+		admin_out(h, "Already disconnected from `%s'", n->name);
 	} else if (nc->type == NETWORK_VIRTUAL && 
 		n->connection.data.virtual.ops->not_disconnectable) {
 		admin_out(h, "Built-in network `%s' can't be disconnected", 
-				  n->info->name);
+				  n->name);
 	} else {
-		admin_out(h, "Disconnecting from `%s'", n->info->name);
+		admin_out(h, "Disconnecting from `%s'", n->name);
 		disconnect_network(n);
 	}
 }
 
-static void cmd_next_server (admin_handle h, char **args, void *userdata) 
+static void cmd_next_server (admin_handle h, const char * const *args, void *userdata) 
 {
 	struct irc_network *n;
 	const char *name;
@@ -315,7 +315,7 @@ static void cmd_next_server (admin_handle h, char **args, void *userdata)
 			admin_out(h, "No current network");
 			return;
 		}
-		name = n->info->name;
+		name = n->name;
 	}
 	if (!n) {
 		admin_out(h, "%s: Not connected", name);
@@ -327,12 +327,16 @@ static void cmd_next_server (admin_handle h, char **args, void *userdata)
 	}
 }
 
-static void cmd_save_config (admin_handle h, char **args, void *userdata)
+static void cmd_save_config (admin_handle h, const char * const *args, void *userdata)
 { 
 	const char *adm_dir;
-	global_update_config(admin_get_global(h));
-	adm_dir = args[1]?args[1]:admin_get_global(h)->config->config_dir; 
-	save_configuration(admin_get_global(h)->config, adm_dir);
+	struct global *global = admin_get_global(h);
+	global_update_config(global);
+	if (global->restricted)
+		adm_dir = global->config->config_dir; 
+	else
+		adm_dir = args[1]?args[1]:global->config->config_dir; 
+	save_configuration(global->config, adm_dir);
 	admin_out(h, "Configuration saved in %s", adm_dir);
 }
 
@@ -353,6 +357,27 @@ static void add_network_helper(admin_handle h, const char *name)
 	admin_out(h, "Network `%s' added. Use ADDSERVER to add a server to this network.", name);
 }
 
+static void rename_network_helper(admin_handle h, const char *oldname, 
+								  const char *newname)
+{
+	struct irc_network *n;
+
+	n = find_network(admin_get_global(h)->networks, oldname);
+
+	if (n == NULL) {
+		admin_out(h, "Network with name `%s' does not exist", oldname);
+		return;
+	}
+
+	if (find_network(admin_get_global(h)->networks, newname) != NULL) {
+		admin_out(h, "Network with name `%s' already exists", newname);
+		return;
+	}
+	
+	g_free(n->name);
+	n->name = g_strdup(newname);
+}
+
 static void del_network_helper(admin_handle h, const char *name)
 {
 	struct irc_network *n = find_network(admin_get_global(h)->networks, name);
@@ -370,6 +395,42 @@ static void del_network_helper(admin_handle h, const char *name)
 	admin_out(h, "Network `%s' deleted", name);
 }
 
+static void info_network_helper(admin_handle h, const char *name)
+{
+	struct irc_network *n = find_network(admin_get_global(h)->networks, name);
+	struct network_config *nc;
+	if (n == NULL) {
+		admin_out(h, "No such network `%s'", name);
+		return;
+	}
+	nc = n->private_data;
+	if (nc->type == NETWORK_TCP) {
+		GList *gl;
+		admin_out(h, "Servers:");
+		for (gl = nc->type_settings.tcp.servers; gl; gl = gl->next) {
+			struct tcp_server_config *server = gl->data;
+			admin_out(h, "  %s:%s%s", server->host, server->port, 
+					  (server == n->connection.data.tcp.current_server?" (Current)":""));
+		}
+	} else if (nc->type == NETWORK_VIRTUAL) {
+		admin_out(h, "Virtual network");
+	} else if (nc->type == NETWORK_PROGRAM) {
+		admin_out(h, "Interface to %s", nc->type_settings.program_location);
+	}
+
+	if (n->clients == NULL) {
+		admin_out(h, "No clients connected");
+	} else {
+		GList *gl;
+		admin_out(h, "Clients:");
+		for (gl = n->clients; gl; gl = gl->next) {
+			struct irc_client *cl = gl->data;
+			admin_out(h, "  %s", cl->description);
+		}
+	}
+}
+
+
 static void list_networks_helper(admin_handle h)
 {
 	GList *gl;
@@ -378,24 +439,24 @@ static void list_networks_helper(admin_handle h)
 
 		switch (n->connection.state) {
 		case NETWORK_CONNECTION_STATE_CONNECTING:
-			admin_out(h, "%s: Connect in progress", n->info->name);
+			admin_out(h, "%s: Connect in progress", n->name);
 			break;
 		case NETWORK_CONNECTION_STATE_CONNECTED:
-			admin_out(h, "%s: Connected, logging in", n->info->name);
+			admin_out(h, "%s: Connected, logging in", n->name);
 			break;
 		case NETWORK_CONNECTION_STATE_NOT_CONNECTED:
 			if (n->connection.data.tcp.last_disconnect_reason)
-				admin_out(h, "%s: Not connected: %s", n->info->name, 
+				admin_out(h, "%s: Not connected: %s", n->name, 
 						  n->connection.data.tcp.last_disconnect_reason);
 			else
-				admin_out(h, "%s: Not connected", n->info->name);
+				admin_out(h, "%s: Not connected", n->name);
 			break;
 		case NETWORK_CONNECTION_STATE_RECONNECT_PENDING:
-			admin_out(h, "%s: Reconnecting", n->info->name);
+			admin_out(h, "%s: Reconnecting", n->name);
 			break;
 		case NETWORK_CONNECTION_STATE_LOGIN_SENT:
 		case NETWORK_CONNECTION_STATE_MOTD_RECVD:
-			admin_out(h, "%s: connected", n->info->name);
+			admin_out(h, "%s: connected", n->name);
 			break;
 		}
 	}
@@ -404,7 +465,8 @@ static void list_networks_helper(admin_handle h)
 /* NETWORK LIST */
 /* NETWORK ADD OFTC */
 /* NETWORK DEL OFTC */
-static void cmd_network(admin_handle h, char **args, void *userdata)
+/* NETWORK INFO OFTC */
+static void cmd_network(admin_handle h, const char * const *args, void *userdata)
 {
 	if (args[1] == NULL)
 		goto usage;
@@ -417,6 +479,12 @@ static void cmd_network(admin_handle h, char **args, void *userdata)
 			return;
 		}
 		add_network_helper(h, args[2]);
+	} else if (!strcasecmp(args[1], "rename")) {
+		if (args[2] == NULL || args[3] == NULL) {
+			admin_out(h, "Usage: network rename <oldname> <newname>");
+			return;
+		}
+		rename_network_helper(h, args[2], args[3]);
 	} else if (!strcasecmp(args[1], "del") || 
 			   !strcasecmp(args[1], "delete") ||
 			   !strcasecmp(args[1], "remove")) {
@@ -426,6 +494,13 @@ static void cmd_network(admin_handle h, char **args, void *userdata)
 		}
 		del_network_helper(h, args[2]);
 		return;
+	} else if (!strcasecmp(args[1], "info")) {
+		if (args[2] == NULL) {
+			admin_out(h, "Usage: network info <name>");
+			return;
+		}
+		info_network_helper(h, args[2]);
+		return;
 	} else {
 		goto usage;
 	}
@@ -435,12 +510,12 @@ usage:
 	admin_out(h, "Usage: network [list|add <name>|del <name>]");
 }
 
-static void cmd_list_networks(admin_handle h, char **args, void *userdata)
+static void cmd_list_networks(admin_handle h, const char * const *args, void *userdata)
 {
 	list_networks_helper(h);
 }
 
-static void cmd_detach(admin_handle h, char **args, void *userdata)
+static void cmd_detach(admin_handle h, const char * const *args, void *userdata)
 {
 	struct irc_client *c = admin_get_client(h);
 
@@ -452,7 +527,7 @@ static void cmd_detach(admin_handle h, char **args, void *userdata)
 	client_disconnect(c, "Client exiting");
 }
 
-static void dump_joined_channels(admin_handle h, char **args, void *userdata)
+static void dump_joined_channels(admin_handle h, const char * const *args, void *userdata)
 {
 	struct irc_network *n;
 	GList *gl;
@@ -471,32 +546,32 @@ static void dump_joined_channels(admin_handle h, char **args, void *userdata)
 		}
 	}
 
-	if (!n->state) {
-		admin_out(h, "Network '%s' not connected", n->info->name);
+	if (!n->external_state) {
+		admin_out(h, "Network '%s' not connected", n->name);
 		return;
 	}
 
-	for (gl = n->state->channels; gl; gl = gl->next) {
+	for (gl = n->external_state->channels; gl; gl = gl->next) {
 		struct irc_channel_state *ch = (struct irc_channel_state *)gl->data;
 		admin_out(h, "%s", ch->name);
 	}
 }
 
 #ifdef DEBUG
-static void cmd_abort(admin_handle h, char **args, void *userdata)
+static void cmd_abort(admin_handle h, const char * const *args, void *userdata)
 {
 	abort();
 }
 #endif
 
-static void cmd_die(admin_handle h, char **args, void *userdata)
+static void cmd_die(admin_handle h, const char * const *args, void *userdata)
 {
 	exit(0);
 }
 
 static GHashTable *markers = NULL;
 
-static void cmd_backlog(admin_handle h, char **args, void *userdata)
+static void cmd_backlog(admin_handle h, const char * const *args, void *userdata)
 {
 	struct linestack_marker *lm;
 	struct irc_network *n;
@@ -521,7 +596,7 @@ static void cmd_backlog(admin_handle h, char **args, void *userdata)
 	}
 
 	if (!args[1] || strlen(args[1]) == 0) {
-		admin_out(h, "Sending backlog for network '%s'", n->info->name);
+		admin_out(h, "Sending backlog for network '%s'", n->name);
 
 		linestack_send(n->linestack, lm, NULL, admin_get_client(h),
 					   TRUE, n->global->config->report_time != REPORT_TIME_NEVER,
@@ -607,7 +682,7 @@ static gboolean log_level_set(admin_handle h, const char *value)
 	return TRUE;
 }
 
-static void cmd_charset(admin_handle h, char **args, void *userdata)
+static void cmd_charset(admin_handle h, const char * const *args, void *userdata)
 {
 	struct irc_client *c;
 
@@ -627,7 +702,7 @@ static void cmd_charset(admin_handle h, char **args, void *userdata)
 	}
 }
 
-static void cmd_echo(admin_handle h, char **args, void *userdata)
+static void cmd_echo(admin_handle h, const char * const *args, void *userdata)
 {
 	admin_out(h, "%s", args[1]);
 }
@@ -671,7 +746,7 @@ gboolean process_cmd(admin_handle h, const char *cmd)
 	for (gl = admin_commands; gl; gl = gl->next) {
 		struct admin_command *cmd = (struct admin_command *)gl->data;
 		if (!g_strcasecmp(cmd->name, args[0])) {
-			cmd->handler(h, args, cmd->userdata);
+			cmd->handler(h, (const char * const *)args, cmd->userdata);
 			g_strfreev(args);
 			return TRUE;
 		}
@@ -720,15 +795,14 @@ static gboolean admin_net_init(struct irc_network *n)
 {
 	char *hostmask;
 	char *nicks;
-	struct network_config *nc = n->private_data;
 
 	hostmask = admin_hostmask(n);
 	
-	virtual_network_recv_args(n, n->state->me.hostmask, "JOIN", ADMIN_CHANNEL, NULL);
+	virtual_network_recv_args(n, n->external_state->me.hostmask, "JOIN", ADMIN_CHANNEL, NULL);
 	virtual_network_recv_response(n, RPL_TOPIC, ADMIN_CHANNEL, 
 		"CtrlProxy administration channel | Type `help' for more information",
 							  NULL);
-	nicks = g_strdup_printf("@ctrlproxy %s", nc->nick);
+	nicks = g_strdup_printf("@ctrlproxy %s", n->external_state->me.nick);
 
 	virtual_network_recv_response(n, RPL_NAMREPLY, "=", ADMIN_CHANNEL, nicks, NULL);
 	g_free(nicks);
@@ -745,8 +819,8 @@ static gboolean admin_to_server (struct irc_network *n, struct irc_client *c, co
 		!g_strcasecmp(l->args[0], "NOTICE")) {
 		struct admin_handle ah;
 
-		if (g_strcasecmp(l->args[0], n->state->me.nick) == 0) {
-			virtual_network_recv_args(n, n->state->me.hostmask, l->args[0], l->args[1], NULL);
+		if (g_strcasecmp(l->args[0], n->external_state->me.nick) == 0) {
+			virtual_network_recv_args(n, n->external_state->me.hostmask, l->args[0], l->args[1], NULL);
 			return TRUE;
 		}
 
@@ -775,7 +849,7 @@ static gboolean admin_to_server (struct irc_network *n, struct irc_client *c, co
 
 		for (i = 1; l->args[i]; i++) {
 			if (!g_strcasecmp(l->args[i], "ctrlproxy") ||
-				!g_strcasecmp(l->args[i], n->state->me.nick)) {
+				!g_strcasecmp(l->args[i], n->external_state->me.nick)) {
 				gl = g_list_append(gl, l->args[i]);
 			}
 		}
@@ -797,8 +871,8 @@ static gboolean admin_to_server (struct irc_network *n, struct irc_client *c, co
 			if (!g_strcasecmp(l->args[i], "ctrlproxy")) {
 				gl = g_list_append(gl, g_strdup_printf("%s=+%s@%s", l->args[i], ADMIN_USERNAME, get_my_hostname()));
 			}
-			if (!g_strcasecmp(l->args[i], n->state->me.nick)) {
-				gl = g_list_append(gl, g_strdup_printf("%s=+%s@%s", l->args[i], n->state->me.username, n->state->me.hostname));
+			if (!g_strcasecmp(l->args[i], n->external_state->me.nick)) {
+				gl = g_list_append(gl, g_strdup_printf("%s=+%s@%s", l->args[i], n->external_state->me.username, n->external_state->me.hostname));
 			}
 		}
 
@@ -827,13 +901,13 @@ static gboolean admin_to_server (struct irc_network *n, struct irc_client *c, co
 									  NULL);
 		}
 		if (!strcmp(l->args[1], ADMIN_CHANNEL) ||
-			!strcmp(l->args[1], n->state->me.nick)) {
-			char *fullname = g_strdup_printf("0 %s", n->state->me.fullname);
+			!strcmp(l->args[1], n->external_state->me.nick)) {
+			char *fullname = g_strdup_printf("0 %s", n->external_state->me.fullname);
 			virtual_network_recv_response(n, RPL_WHOREPLY, ADMIN_CHANNEL, 
-									  n->state->me.username,
-									  n->state->me.hostname,
+									  n->external_state->me.username,
+									  n->external_state->me.hostname,
 									  get_my_hostname(),
-									  n->state->me.nick,
+									  n->external_state->me.nick,
 									  "H",
 									  fullname,
 									  NULL);
@@ -852,7 +926,7 @@ static gboolean admin_to_server (struct irc_network *n, struct irc_client *c, co
 		if (strcmp(l->args[1], ADMIN_CHANNEL) != 0) {
 			virtual_network_recv_response(n, ERR_NOTONCHANNEL, l->args[1], "You're not on that channel", NULL);
 		} else {
-			virtual_network_recv_args(n, n->state->me.hostmask, "PART", l->args[1], NULL);
+			virtual_network_recv_args(n, n->external_state->me.hostmask, "PART", l->args[1], NULL);
 			admin_net_init(n);
 		}
 		return TRUE;
@@ -907,7 +981,7 @@ void admin_log(enum log_level level, const struct irc_network *n, const struct i
 	tmp = g_strdup_printf("%s%s%s%s%s%s", 
 						  data, 
 						  n?" (":"",
-						  n?n->info->name:"", 
+						  n?n->name:"", 
 						  c?"/":"",
 						  c?c->description:"",
 						  n?")":"");
@@ -932,7 +1006,7 @@ void admin_log(enum log_level level, const struct irc_network *n, const struct i
 	entered = FALSE;
 }
 
-static void cmd_start_listener(admin_handle h, char **args, void *userdata)
+static void cmd_start_listener(admin_handle h, const char * const *args, void *userdata)
 {
 	char *b, *p;
 	struct listener_config *cfg;
@@ -969,20 +1043,21 @@ static void cmd_start_listener(admin_handle h, char **args, void *userdata)
 		}
 	}
 
+	cfg->password = g_strdup(args[2]);
+
 	l = listener_init(admin_get_global(h), cfg);
-	l->config->password = g_strdup(args[2]);
 	l->global = admin_get_global(h);
 
-	listener_start(l, l->config->address, l->config->port);
+	listener_start_tcp(l, l->config->address, l->config->port);
 }
 
-static void cmd_stop_listener(admin_handle h, char **args, void *userdata)
+static void cmd_stop_listener(admin_handle h, const char * const *args, void *userdata)
 {
 	GList *gl;
 	char *b, *p;
 	int i = 0;
 
-	if (!args[0]) {
+	if (args[1] == NULL) {
 		admin_out(h, "No port specified");
 		return;
 	}
@@ -1018,7 +1093,7 @@ static void cmd_stop_listener(admin_handle h, char **args, void *userdata)
 	admin_out(h, "%d listeners stopped", i);
 }
 
-static void cmd_list_listener(admin_handle h, char **args, void *userdata)
+static void cmd_list_listener(admin_handle h, const char * const *args, void *userdata)
 {
 	GList *gl;
 
@@ -1026,7 +1101,7 @@ static void cmd_list_listener(admin_handle h, char **args, void *userdata)
 		struct irc_listener *l = gl->data;
 
 		admin_out(h, "%s:%s%s%s%s", l->config->address?l->config->address:"", l->config->port, 
-				  l->network?" (":"", l->network?l->network->info->name:"", 
+				  l->network?" (":"", l->network?l->network->name:"", 
 				  l->network?")":"");
 	}
 }
@@ -1172,17 +1247,17 @@ static char *max_who_age_get(admin_handle h)
 {
 	struct global *g = admin_get_global(h);
 
-	if (g->config->max_who_age == 0)
+	if (g->config->cache.max_who_age == 0)
 		return NULL;
 	
-	return g_strdup_printf("%d", g->config->max_who_age);
+	return g_strdup_printf("%d", g->config->cache.max_who_age);
 }
 
 static gboolean max_who_age_set(admin_handle h, const char *value)
 {
 	struct global *g = admin_get_global(h);
 
-	g->config->max_who_age = (value == NULL?0:atoi(value));
+	g->config->cache.max_who_age = (value == NULL?0:atoi(value));
 
 	return TRUE;
 }
@@ -1258,6 +1333,22 @@ static gboolean auto_away_nick_set(admin_handle h, const char *value)
 	return TRUE;
 }
 
+static char *auto_away_client_limit_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+	
+	return g_strdup_printf("%d", g->config->auto_away.client_limit);
+}
+
+static gboolean auto_away_client_limit_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+
+	g->config->auto_away.client_limit = (value == NULL?-1:atoi(value));
+
+	return TRUE;
+}
+
 static char *report_time_offset_get(admin_handle h)
 {
 	struct global *g = admin_get_global(h);
@@ -1433,6 +1524,81 @@ static gboolean default_client_charset_set(admin_handle h, const char *value)
 	return TRUE;
 }
 
+static char *default_nick_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+	return g_strdup(g->config->default_nick);
+}
+
+static gboolean default_nick_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+	if (strlen(value) == 0) {
+		admin_out(h, "Nick name can't be empty.");
+		return FALSE;
+	}
+	if (strchr(value, ' ') != NULL) {
+		admin_out(h, "Nick name can't contain spaces.");
+		return FALSE;
+	}
+	if (value[0] == ':') {
+		admin_out(h, "Nick name can't start with semicolon.");
+		return FALSE;
+	}
+
+	g_free(g->config->default_nick);
+	g->config->default_nick = g_strdup(value);
+	return TRUE;
+}
+
+
+static char *default_username_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+	return g_strdup(g->config->default_username);
+}
+
+static gboolean default_username_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+	if (strlen(value) == 0) {
+		admin_out(h, "Username can't be empty.");
+		return FALSE;
+	}
+	if (strchr(value, ' ') != NULL) {
+		admin_out(h, "Username can't contain spaces.");
+		return FALSE;
+	}
+	if (value[0] == ':') {
+		admin_out(h, "Username can't start with semicolon.");
+		return FALSE;
+	}
+
+	g_free(g->config->default_username);
+	g->config->default_username = g_strdup(value);
+	return TRUE;
+}
+
+
+static char *default_fullname_get(admin_handle h)
+{
+	struct global *g = admin_get_global(h);
+	return g_strdup(g->config->default_realname);
+}
+
+static gboolean default_fullname_set(admin_handle h, const char *value)
+{
+	struct global *g = admin_get_global(h);
+
+	if (strlen(value) == 0) {
+		admin_out(h, "Fullname can't be empty");
+		return FALSE;
+	}
+
+	g_free(g->config->default_realname);
+	g->config->default_realname = g_strdup(value);
+	return TRUE;
+}
 
 /**
  * Table of administration settings that can be
@@ -1446,6 +1612,7 @@ static struct admin_setting {
 	{ "admin-log", admin_log_get, admin_log_set },
 	{ "admin-user", admin_user_get, admin_user_set },
 	{ "auto-away-enable", auto_away_enable_get, auto_away_enable_set },
+	{ "auto-away-client-limit", auto_away_client_limit_get, auto_away_client_limit_set },
 	{ "auto-away-message", auto_away_message_get, auto_away_message_set },
 	{ "auto-away-nick", auto_away_nick_get, auto_away_nick_set },
 	{ "auto-away-time", auto_away_time_get, auto_away_time_set },
@@ -1465,10 +1632,13 @@ static struct admin_setting {
 	{ "report-time", report_time_get, report_time_set },
 	{ "report-time-offset", report_time_offset_get, report_time_offset_set },
 	{ "replication", replication_get, replication_set },
+	{ "default-nick", default_nick_get, default_nick_set },
+	{ "default-username", default_username_get, default_username_set },
+	{ "default-fullname", default_fullname_get, default_fullname_set },
 	{ NULL, NULL, NULL }
 };
 
-static void cmd_unset(admin_handle h, char **args, void *userdata)
+static void cmd_unset(admin_handle h, const char * const *args, void *userdata)
 {
 	int i;
 	if (args[1] == NULL) {
@@ -1483,7 +1653,7 @@ static void cmd_unset(admin_handle h, char **args, void *userdata)
 	}
 }
 
-static void cmd_set(admin_handle h, char **args, void *userdata)
+static void cmd_set(admin_handle h, const char * const *args, void *userdata)
 {
 	int i;
 	char *tmp;
