@@ -21,10 +21,6 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include <netinet/in.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -40,6 +36,21 @@ static gboolean process_to_client(struct irc_client *c, const struct irc_line *l
 	return TRUE;
 }
 
+static void handle_offline_command(struct irc_client *c, const struct irc_line *l, const char *offline_reason)
+{
+	if (!g_strcasecmp(l->args[0], "PRIVMSG") || !g_strcasecmp(l->args[0], "NOTICE")) {
+		client_send_response(c, ERR_NOSUCHNICK, l->args[1], offline_reason, NULL);
+	} else if (!g_strcasecmp(l->args[0], "JOIN")) {
+		/* Make network->internal_state join channel */
+		client_send_response(c, ERR_NOSUCHCHANNEL, l->args[1], offline_reason, NULL);
+	} else if (!g_strcasecmp(l->args[0], "PART")) {
+		/* Make network->internal_state part channel */
+		client_send_response(c, ERR_NOTONCHANNEL, l->args[1], offline_reason, NULL);
+	} else {
+		client_send_response(c, ERR_UNKNOWNCOMMAND, l->args[0], offline_reason, NULL);
+	}
+}
+
 /**
  * Process incoming lines from a client.
  *
@@ -72,7 +83,7 @@ static gboolean process_from_client(struct irc_client *c, const struct irc_line
 		g_free(l->origin);
 		return FALSE;
 	} else if (!g_strcasecmp(l->args[0], "PING")) {
-		client_send_args(c, "PONG", c->network->info->name, l->args[1], NULL);
+		client_send_args(c, "PONG", c->network->name, l->args[1], NULL);
 	} else if (!g_strcasecmp(l->args[0], "PONG")) {
 		if (l->argc < 2) {
 			client_send_response(c, ERR_NEEDMOREPARAMS, l->args[0], 
@@ -105,9 +116,9 @@ static gboolean process_from_client(struct irc_client *c, const struct irc_line
 	} else if (c->network->connection.state == NETWORK_CONNECTION_STATE_MOTD_RECVD) {
 		struct network_config *nc = c->network->private_data;
 
-		if (nc->disable_cache || !client_try_cache(c, c->network->state, l)) {
+		if (nc->disable_cache || !client_try_cache(c, c->network->external_state, l, &c->network->global->config->cache)) {
 			/* Perhaps check for validity of input here ? It could save us some bandwidth 
-			 * to the server, though very unlikely to occur often */
+			 * to the server, though unlikely to occur often */
 			network_send_line(c->network, c, l, FALSE);
 		}
 	} else if (c->network->connection.state == NETWORK_CONNECTION_STATE_NOT_CONNECTED) {
@@ -118,7 +129,8 @@ static gboolean process_from_client(struct irc_client *c, const struct irc_line
 			msg = g_strdup_printf("Currently not connected to server... (%s)",
 					c->network->connection.data.tcp.last_disconnect_reason);
 
-		client_send_args(c, "NOTICE", client_get_default_target(c), msg, NULL);
+		handle_offline_command(c, l, msg);
+
 		g_free(msg);
 	}
 
@@ -147,10 +159,10 @@ static gboolean welcome_client(struct irc_client *client)
 	client_send_response(client, RPL_CREATED, 
 		"Ctrlproxy (c) 2002-2008 Jelmer Vernooij <jelmer@samba.org>", NULL);
 	client_send_response(client, RPL_MYINFO, 
-		 client->network->info->name, 
+		 client->network->name, 
 		 ctrlproxy_version(), 
-		 (client->network->state != NULL && client->network->info->supported_user_modes)?client->network->info->supported_user_modes:ALLMODES, 
-		 (client->network->state != NULL && client->network->info->supported_channel_modes)?client->network->info->supported_channel_modes:ALLMODES,
+		 (client->network->external_state != NULL && client->network->info->supported_user_modes)?client->network->info->supported_user_modes:ALLMODES, 
+		 (client->network->external_state != NULL && client->network->info->supported_channel_modes)?client->network->info->supported_channel_modes:ALLMODES,
 		 NULL);
 
 	features = network_generate_feature_string(client->network);
@@ -160,8 +172,8 @@ static gboolean welcome_client(struct irc_client *client)
 
 	g_free(features);
 
-	if (client->network->state != NULL) {
-		client_send_luserchannels(client, g_list_length(client->network->state->channels));
+	if (client->network->external_state != NULL) {
+		client_send_luserchannels(client, g_list_length(client->network->external_state->channels));
 	}
 
 	tmp = g_strdup_printf("I have %d clients", g_list_length(client->network->clients));
@@ -181,17 +193,17 @@ static gboolean welcome_client(struct irc_client *client)
 	g_assert(client->state != NULL);
 	g_assert(client->network != NULL);
 
-	if (client->network->state != NULL) {
-		if (g_strcasecmp(client->state->me.nick, client->network->state->me.nick) != 0) {
+	if (client->network->external_state != NULL) {
+		if (g_strcasecmp(client->state->me.nick, client->network->external_state->me.nick) != 0) {
 			/* Tell the client our his/her real nick */
 			client_send_args_ex(client, client->state->me.hostmask, "NICK", 
-								client->network->state->me.nick, NULL); 
+								client->network->external_state->me.nick, NULL); 
 
 			/* Try to get the nick the client specified */
 			nc = client->network->private_data;
 			if (!nc->ignore_first_nick) {
 				network_send_args(client->network, "NICK", 
-								  client->requested_nick, NULL);
+								  client->login_details->nick, NULL);
 			}
 		}
 	}
@@ -217,6 +229,33 @@ static void handle_client_disconnect(struct irc_client *c)
 		c->network->clients = g_list_remove(c->network->clients, c);
 }
 
+static struct irc_network *client_connect_command(struct irc_client *client, const char *hostname, guint16 port)
+{
+	extern struct global *my_global;
+
+	struct irc_network *network;
+
+	network = network_ref(find_network_by_hostname(my_global, 
+												   hostname, port, my_global->config->create_implicit, 
+												   client->login_details));
+
+	if (network == NULL) {
+		client_log(LOG_ERROR, client, 
+			"Unable to connect to network with name %s", 
+			hostname);
+		return NULL;
+	}
+
+	if (network->connection.state == NETWORK_CONNECTION_STATE_NOT_CONNECTED) {
+		client_send_args(client, "NOTICE", 
+							client_get_default_target(client),
+							"Connecting to network", NULL);
+		connect_network(network);
+	}
+
+	return network;
+}
+
 void log_client(enum log_level, const struct irc_client *, const char *data);
 static struct irc_client_callbacks default_callbacks = {
 	.process_from_client = process_from_client, 
@@ -225,23 +264,30 @@ static struct irc_client_callbacks default_callbacks = {
 	.disconnect = handle_client_disconnect,
 	.free_private_data = client_free_private,
 	.welcome = welcome_client,
+	.on_connect = client_connect_command,
 };
 
 struct irc_client *client_init_iochannel(struct irc_network *n, GIOChannel *c, const char *desc)
 {
 	struct irc_transport *transport;
+	struct irc_client *client;
 	g_io_channel_set_flags(c, G_IO_FLAG_NONBLOCK, NULL);
 	g_io_channel_set_close_on_unref(c, TRUE);
 	transport = irc_transport_new_iochannel(c);
 
-	return client_init(n, transport, desc);
+	client = client_init(n, transport, desc);
+
+	transport_parse_buffer(transport);
+
+	return client;
 }
 
 struct irc_client *client_init(struct irc_network *n, struct irc_transport *transport, const char *desc)
 {
 	struct irc_client *client;
 
-	client = irc_client_new(transport, n?n->info->name:get_my_hostname(), desc, &default_callbacks);
+	client = irc_client_new(transport, n?n->name:get_my_hostname(), desc, &default_callbacks);
+	client->authenticated = FALSE;
 
 	client->network = network_ref(n);
 
