@@ -21,15 +21,22 @@
 #include <string.h>
 #include <fcntl.h>
 #include <stdio.h>
-#include <unistd.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include <errno.h>
 
 #include <glib/gstdio.h>
 #include <sys/stat.h>
 
+#define README_CONTENTS  \
+"This directory contains the history information that ctrlproxy uses\n" \
+"when sending backlogs.\n" \
+"\n" \
+"It is safe to remove this data while ctrlproxy is not running and \n" \
+"will not break your configuration. \n" \
+"\n" \
+"If you delete it while ctrlproxy is running, you will lose the ability \n" \
+"to use any backlog functionality during the current session.\n"
+
+
 /*
  * Marshall/unmarshall functions
  */
@@ -182,28 +189,6 @@ static gboolean marshall_char(struct irc_network_state *nst, const char *name, i
 	}
 }
 
-static gboolean marshall_long(struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, long *n)
-{
-	if (m == MARSHALL_PUSH) {
-		char tmp[10];
-		GError *error = NULL;
-		GIOStatus status;
-		g_snprintf(tmp, sizeof(tmp), "%ld", *n);
-		status = marshall_set(t, level, name, tmp, &error);
-		return (status == G_IO_STATUS_NORMAL);
-	} else {
-		char *tmp;
-		gboolean ret = marshall_get(t, level, name, &tmp);
-		if (!ret) return FALSE;
-
-		*n = atol(tmp);
-
-		g_free(tmp);
-
-		return TRUE;
-	}
-}
-
 static gboolean marshall_int(struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, int *n)
 {
 	if (m == MARSHALL_PUSH) {
@@ -232,7 +217,7 @@ static gboolean marshall_time(struct irc_network_state *nst, const char *name, i
 		char tmp[30];
 		GError *error = NULL;
 		GIOStatus status;
-		g_snprintf(tmp, sizeof(tmp), "%lu", *n);
+		g_snprintf(tmp, sizeof(tmp), "%ld", *n);
 		status = marshall_set(t, level, name, tmp, &error);
 		return (status == G_IO_STATUS_NORMAL);
 	} else {
@@ -240,7 +225,7 @@ static gboolean marshall_time(struct irc_network_state *nst, const char *name, i
 		gboolean ret = marshall_get(t, level, name, &tmp);
 		if (!ret) return FALSE;
 
-		*n = strtoul(tmp, NULL, 10);
+		*n = strtol(tmp, NULL, 0);
 
 		g_free(tmp);
 
@@ -338,7 +323,7 @@ static gboolean marshall_GList (struct irc_network_state *nst, const char *name,
 	return TRUE;
 }
 
-static gboolean marshall_banlist_entry (struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, struct banlist_entry **d)
+static gboolean marshall_nicklist_entry (struct irc_network_state *nst, const char *name, int level, enum marshall_mode m, GIOChannel *t, struct nicklist_entry **d)
 {
 	gboolean ret = TRUE;
 	marshall_new(m, d);
@@ -354,6 +339,7 @@ static gboolean marshall_channel_state (struct irc_network_state *nst,
 										enum marshall_mode m, GIOChannel *t, 
 										struct irc_channel_state **c)
 {
+	int i;
 	gboolean ret = TRUE;
 	marshall_new(m, c);
 	marshall_struct(t, m, level, name);
@@ -364,15 +350,17 @@ static gboolean marshall_channel_state (struct irc_network_state *nst,
 	ret &= marshall_bool(nst, "banlist_started", level+1, m, t, &(*c)->banlist_started);
 	ret &= marshall_bool(nst, "invitelist_started", level+1, m, t, &(*c)->invitelist_started);
 	ret &= marshall_bool(nst, "exceptlist_started", level+1, m, t, &(*c)->exceptlist_started);
-	ret &= marshall_int(nst, "limit", level+1, m, t, &(*c)->limit);
 	ret &= marshall_string(nst, "name", level+1, m, t, &(*c)->name);
-	ret &= marshall_string(nst, "key", level+1, m, t, &(*c)->key);
 	ret &= marshall_string(nst, "topic", level+1, m, t, &(*c)->topic);
 	ret &= marshall_string(nst, "topic_set_by", level+1, m, t, &(*c)->topic_set_by);
-	ret &= marshall_long(nst, "topic_set_time", level+1, m, t, &(*c)->topic_set_time);
-	ret &= marshall_GList(nst, "banlist", level+1, m, t, &(*c)->banlist, (marshall_fn_t)marshall_banlist_entry);
-	ret &= marshall_GList(nst, "invitelist", level+1, m, t, &(*c)->invitelist, (marshall_fn_t)marshall_string);
-	ret &= marshall_GList(nst, "exceptlist", level+1, m, t, &(*c)->exceptlist, (marshall_fn_t)marshall_string);
+	ret &= marshall_time(nst, "topic_set_time", level+1, m, t, &(*c)->topic_set_time);
+	for (i = 0; i < MAXMODES; i++) {
+		char name[20];
+		g_snprintf(name, sizeof(name), "mode_nicklist[%d]", i);
+		ret &= marshall_GList(nst, name, level+1, m, t, &(*c)->chanmode_nicklist[i], (marshall_fn_t)marshall_nicklist_entry);
+		g_snprintf(name, sizeof(name), "mode_option[%d]", i);
+		ret &= marshall_string(nst, name, level+1, m, t, &(*c)->chanmode_option[i]);
+	}
 	(*c)->network = nst;
 
 	marshall_struct(t, m, level+1, "nicks");
@@ -477,6 +465,22 @@ static gboolean file_insert_state(struct linestack_context *ctx,
 							  const struct irc_network_state *state, 
 							  const char *state_id);
 
+static char *global_init(struct ctrlproxy_config *config)
+{
+	char *parent_dir, *readme_file;;
+	parent_dir = g_build_filename(config->config_dir, "linestack_file", NULL);
+	if (g_file_test(parent_dir, G_FILE_TEST_IS_DIR))
+		return parent_dir;
+	g_mkdir(parent_dir, 0700);
+	readme_file = g_build_filename(parent_dir, "README", NULL);
+	if (!g_file_test(readme_file, G_FILE_TEST_EXISTS)) {
+		g_file_set_contents(readme_file, README_CONTENTS, 
+							-1, NULL);
+	}
+	g_free(readme_file);
+	return parent_dir;
+}
+
 static gboolean file_init(struct linestack_context *ctx, const char *name, 
 						  struct ctrlproxy_config *config, 
 						  const struct irc_network_state *state)
@@ -487,8 +491,8 @@ static gboolean file_init(struct linestack_context *ctx, const char *name,
 	const char *fname;
 	GDir *dir;
 
-	parent_dir = g_build_filename(config->config_dir, "linestack_file", NULL);
-	g_mkdir(parent_dir, 0700);
+	parent_dir = global_init(config);
+
 	data_dir = g_build_filename(parent_dir, name, NULL);
 	g_free(parent_dir);
 	g_mkdir(data_dir, 0700);
