@@ -27,6 +27,12 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <glib.h>
+#include <sys/un.h>
+
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
 
 void default_listener_log_fn(enum log_level l, const struct irc_listener *listener, const char *ret)
 {
@@ -36,36 +42,57 @@ void default_listener_log_fn(enum log_level l, const struct irc_listener *listen
 		log_global(l, "%s", ret);
 }
 
-gboolean default_socks_auth_simple(struct pending_client *cl, const char *username, const char *password)
+#ifdef HAVE_GSSAPI
+gboolean default_socks_gssapi(struct pending_client *cl, gss_name_t authn_name)
+{
+	/* FIXME: Check if principal matches own user name */
+	return FALSE;
+}
+#endif
+
+static gboolean listener_check_username_password(struct irc_listener *listener, 
+												 const char *username, const char *password)
 {
 	GList *gl;
 
-	for (gl = cl->listener->config->allow_rules; gl; gl = gl->next)
-	{
-		struct allow_rule *r = gl->data;
+	if (listener->config != NULL) {
+		for (gl = listener->config->allow_rules; gl; gl = gl->next)
+		{
+			struct allow_rule *r = gl->data;
 
-		if (r->password == NULL || r->username == NULL) 
-			continue;
+			if (r->password == NULL || r->username == NULL) 
+				continue;
 
-		if (strcmp(r->username, username)) 
-			continue;
+			if (strcmp(r->username, username)) 
+				continue;
 
-		if (strcmp(r->password, password))
-			continue;
+			if (strcmp(r->password, password))
+				continue;
 
-		return TRUE;
+			return TRUE;
+		}
+
+		if (listener->config->password != NULL) {
+			return (strcmp(listener->config->password, password) == 0);
+		}
 	}
 
-	if (cl->listener->config->password == NULL) {
-		listener_log(LOG_WARNING, cl->listener, "No password set, allowing client _without_ authentication!");
-		return TRUE;
+	if (listener->global->config->password != NULL) {
+		return (strcmp(listener->global->config->password, password) == 0);
 	}
 
-	if (strcmp(cl->listener->config->password, password) == 0) {
-		return TRUE;
-	}
+	listener_log(LOG_WARNING, listener, "No password set, allowing client _without_ authentication!");
+	return TRUE;
+}
 
-	return FALSE;
+
+gboolean default_socks_auth_simple(struct pending_client *cl, const char *username, const char *password,
+								   gboolean (*on_known)(struct pending_client *, gboolean))
+{
+	if (listener_check_username_password(cl->listener, username, password))
+		return on_known(cl, TRUE);
+
+	return on_known(cl, FALSE);
 }
 
 gboolean default_socks_connect_fqdn (struct pending_client *cl, const char *hostname, uint16_t port)
@@ -77,7 +104,7 @@ gboolean default_socks_connect_fqdn (struct pending_client *cl, const char *host
 	listener_log(LOG_INFO, cl->listener, "Request to connect to %s:%d", hostname, port);
 
 	result = find_network_by_hostname(cl->listener->global, hostname, port, 
-									  cl->listener->global->config->create_implicit);
+									  cl->listener->global->config->create_implicit, NULL);
 
 	if (result == NULL) {
 		listener_log(LOG_WARNING, cl->listener, "Unable to return network matching %s:%d", hostname, port);
@@ -126,6 +153,12 @@ gboolean default_socks_connect_fqdn (struct pending_client *cl, const char *host
 	}
 
 	desc = g_io_channel_ip_get_description(cl->connection);
+	if (desc == NULL) {
+		if (cl->listener == cl->listener->global->unix_domain_socket_listener) 
+			desc = g_strdup("Unix domain socket socks client");
+		else
+			desc = g_strdup("socks client");
+	}
 	client_init_iochannel(result, cl->connection, desc);
 	g_free(desc);
 
@@ -155,37 +188,41 @@ void listener_log(enum log_level l, const struct irc_listener *listener,
 
 static gboolean handle_client_line(struct pending_client *pc, const struct irc_line *l)
 {
+	struct irc_listener *listener = pc->listener;
+
 	if (l == NULL || l->args[0] == NULL) { 
 		return TRUE;
 	}
 
 	if (!g_strcasecmp(l->args[0], "PASS")) {
 		const char *networkname = NULL;
-		struct irc_network *n = pc->listener->network;
+		struct irc_network *n = listener->network;
 		gboolean authenticated = FALSE;
+		char *actual_password;
 
-		if (pc->listener->config->password == NULL) {
-			listener_log(LOG_WARNING, pc->listener,
-							"No password set, allowing client _without_ authentication!");
-			authenticated = TRUE;
-			networkname = l->args[1];
-		} else if (strcmp(l->args[1], pc->listener->config->password) == 0) {
-			authenticated = TRUE;
-		} else if (strncmp(l->args[1], pc->listener->config->password, 
-						   strlen(pc->listener->config->password)) == 0 &&
-				   l->args[1][strlen(pc->listener->config->password)] == ':') {
-			authenticated = TRUE;
-			networkname = l->args[1]+strlen(pc->listener->config->password)+1;
+		if (strchr(l->args[1], ':') != NULL) {
+			char *p = strchr(l->args[1], ':');
+			actual_password = g_strndup(l->args[1], p-l->args[1]);
+			networkname = p+1;
+		} else {
+			actual_password = g_strdup(l->args[1]);
+			networkname = NULL;
 		}
 
+		authenticated = listener_check_username_password(listener, NULL, actual_password);
+
+		g_free(actual_password);
+
 		if (authenticated) {
-			listener_log(LOG_INFO, pc->listener, "Client successfully authenticated");
+			listener_log(LOG_INFO, listener, "Client successfully authenticated");
 
 			if (networkname != NULL) {
-				n = find_network_by_hostname(pc->listener->global, 
-											 networkname, 6667, pc->listener->global->config->create_implicit);
+				n = find_network_by_hostname(listener->global, 
+											 networkname, 6667, listener->global->config->create_implicit,
+											 NULL);
 				if (n == NULL) {
-					irc_sendf(pc->connection, pc->listener->iconv, NULL, ":%s %d %s :Password error: unable to find network", 
+					irc_sendf(pc->connection, listener->iconv, NULL, 
+							  ":%s %d %s :Password error: unable to find network", 
 							  get_my_hostname(), ERR_PASSWDMISMATCH, "*");
 					g_io_channel_flush(pc->connection, NULL);
 					return FALSE;
@@ -193,28 +230,39 @@ static gboolean handle_client_line(struct pending_client *pc, const struct irc_l
 
 				if (n->connection.state == NETWORK_CONNECTION_STATE_NOT_CONNECTED && 
 					!connect_network(n)) {
-					irc_sendf(pc->connection, pc->listener->iconv, NULL, ":%s %d %s :Password error: unable to connect", 
+					irc_sendf(pc->connection, listener->iconv, NULL, 
+							  ":%s %d %s :Password error: unable to connect", 
 							  get_my_hostname(), ERR_PASSWDMISMATCH, "*");
 					g_io_channel_flush(pc->connection, NULL);
 					return FALSE;
 				}
 			}
 
+			irc_sendf(pc->connection, listener->iconv, NULL, 
+					  "NOTICE AUTH :PASS OK");
+			g_io_channel_flush(pc->connection, NULL);
+
+
 			{
 				char *desc = g_io_channel_ip_get_description(pc->connection);
-				client_init_iochannel(pc->listener->network, pc->connection, desc);
+				if (pc->listener == pc->listener->global->unix_domain_socket_listener) 
+					desc = g_strdup("Unix domain socket client");
+				else if (desc == NULL)
+					desc = g_strdup("");
+				client_init_iochannel(n, pc->connection, desc);
 				g_free(desc);
 			}
 
 			return FALSE;
 		} else {
 			GIOStatus status;
-			listener_log(LOG_WARNING, pc->listener, 
+			listener_log(LOG_WARNING, listener, 
 						 "User tried to log in with incorrect password!");
 
-			status = irc_sendf(pc->connection, pc->listener->iconv, NULL, 
+			status = irc_sendf(pc->connection, listener->iconv, NULL, 
 							   ":%s %d %s :Password mismatch", 
 							   get_my_hostname(), ERR_PASSWDMISMATCH, "*");
+			g_io_channel_flush(pc->connection, NULL);
 
 			if (status != G_IO_STATUS_NORMAL) {
 				return FALSE;
@@ -223,7 +271,7 @@ static gboolean handle_client_line(struct pending_client *pc, const struct irc_l
 			return TRUE;
 		}
 	} else {
-		irc_sendf(pc->connection, pc->listener->iconv, NULL, ":%s %d %s :You are not registered", get_my_hostname(), ERR_NOTREGISTERED, "*");
+		irc_sendf(pc->connection, listener->iconv, NULL, ":%s %d %s :You are not registered. Did you specify a password?", get_my_hostname(), ERR_NOTREGISTERED, "*");
 		g_io_channel_flush(pc->connection, NULL);
 	}
 
@@ -248,6 +296,9 @@ void free_listener(struct irc_listener *l)
 static struct irc_listener_ops default_listener_ops = {
 	.handle_client_line = handle_client_line,
 	.socks_auth_simple = default_socks_auth_simple,
+#ifdef HAVE_GSSAPI
+	.socks_gssapi = default_socks_gssapi,
+#endif
 	.socks_connect_fqdn = default_socks_connect_fqdn,
 };
 
@@ -258,6 +309,7 @@ struct irc_listener *listener_init(struct global *global, struct listener_config
 	l->config = cfg;
 	l->ssl = cfg->ssl;
 	l->ssl_credentials = cfg->ssl_credentials;
+	g_assert(!l->ssl || l->ssl_credentials != NULL);
 	l->global = global;
 	l->ops = &default_listener_ops;
 	l->iconv = (GIConv)-1;
@@ -294,7 +346,7 @@ static void auto_add_listener(struct irc_network *n, void *private_data)
 	cfg->network = g_strdup(nc->name);
 	cfg->port = g_strdup_printf("%d", next_autoport(n->global));
 	l = listener_init(n->global, cfg);
-	listener_start(l, NULL, cfg->port);
+	listener_start_tcp(l, NULL, cfg->port);
 }
 
 gboolean init_listeners(struct global *global)
@@ -310,7 +362,7 @@ gboolean init_listeners(struct global *global)
 		struct irc_listener *l = listener_init(global, cfg);
 
 		if (l != NULL) {
-			ret &= listener_start(l, cfg->address, cfg->port);
+			ret &= listener_start_tcp(l, cfg->address, cfg->port);
 		}
 	}
 	return ret;
@@ -327,4 +379,62 @@ void fini_listeners(struct global *global)
 	}
 }
 
+gboolean start_unix_domain_socket_listener(struct global *global)
+{
+	int sock;
+	struct sockaddr_un un;
+	struct irc_listener *l = g_new0(struct irc_listener, 1);
+	GIOChannel *ioc;
 
+	sock = socket(PF_UNIX, SOCK_STREAM, 0);
+	if (sock < 0) {
+		log_global(LOG_ERROR, "error creating unix socket: %s", strerror(errno));
+		g_free(l);
+		return FALSE;
+	}
+	
+	un.sun_family = AF_UNIX;
+	strncpy(un.sun_path, global->config->network_socket, sizeof(un.sun_path));
+	unlink(un.sun_path);
+
+	if (bind(sock, (struct sockaddr *)&un, sizeof(un)) < 0) {
+		log_global(LOG_ERROR, "unable to bind to %s: %s", un.sun_path, strerror(errno));
+		g_free(l);
+		return FALSE;
+	}
+	
+	if (listen(sock, 5) < 0) {
+		log_global(LOG_ERROR, "error listening on socket: %s", strerror(errno));
+		g_free(l);
+		return FALSE;
+	}
+
+	l->ops = &default_listener_ops;
+	l->config = NULL;
+	l->iconv = (GIConv)-1;
+	l->ssl = FALSE;
+	l->ssl_credentials = NULL;
+	l->global = global;
+	l->log_fn = default_listener_log_fn;
+
+	ioc = g_io_channel_unix_new(sock);
+
+	if (ioc == NULL) {
+		log_global(LOG_ERROR, "Unable to create GIOChannel for unix server socket");
+		return FALSE;
+	}
+
+	listener_add_iochannel(l, ioc, NULL, NULL);
+
+	global->unix_domain_socket_listener = l;
+
+	return TRUE;
+}
+
+gboolean stop_unix_domain_socket_listener(struct global *global)
+{
+	if (global->unix_domain_socket_listener != NULL)
+		listener_stop(global->unix_domain_socket_listener);
+	unlink(global->config->network_socket);
+	return TRUE;
+}
