@@ -30,12 +30,11 @@
 #include <sys/socket.h>
 #include <glib/gstdio.h>
 
-#define DEFAULT_ADMIN_PORT 6680
+#define DEFAULT_ADMIN_PORT 57000
 #define DEFAULT_SOCKS_PORT 1080
 
 #define CHANNEL_KEY_FILE_HEADER \
 	"; This file contains channel keys.\n" \
-	"; It has the same format as the ppp secrets files.\n" \
 	"; It should contain one entry per line, each entry consisting of: \n" \
 	"; a channel name, channel key and network name, separated by tabs.\n" \
 	";\n\n"
@@ -57,6 +56,7 @@ static const char *builtin_known_keys[] = {
 	"auto-away-client-limit",
 	"auto-away-time",
 	"create-implicit",
+	"max-who-age",
 	"max_who_age",
 	"replication",
 	"linestack",
@@ -69,6 +69,9 @@ static const char *builtin_known_keys[] = {
 	"admin-log",
 	"admin-user",
 	"password",
+	"default-username",
+	"default-nick",
+	"default-fullname",
 	"listener-auto",
 	"listener-autoport",
 	"logging",
@@ -77,6 +80,7 @@ static const char *builtin_known_keys[] = {
 	"port",
 	"bind",
 	"ssl",
+	"keytab",
 	"default-network",
 	"log-logfilename",
 	"log-format-nickchange",
@@ -140,7 +144,7 @@ gboolean g_key_file_save_to_file(GKeyFile *kf, const gchar *file, GError **error
 	return TRUE;
 }
 
-static void config_save_tcp_servers(struct network_config *n, GKeyFile *kf)
+static void config_save_tcp_servers(struct network_config *n, GKeyFile *kf, const char *groupname)
 {
 	GList *gl;
 	int i = 0; 
@@ -165,54 +169,64 @@ static void config_save_tcp_servers(struct network_config *n, GKeyFile *kf)
 		i++;
 	}
 
-	g_key_file_set_string_list(kf, "global", "servers", (const gchar **)values, i);
+	g_key_file_set_string_list(kf, groupname, "servers", (const gchar **)values, i);
 
 	g_strfreev(values);
 }
 
-static void config_save_network(const char *dir, struct network_config *n, GList **channel_keys)
+static void config_save_network(struct ctrlproxy_config *cfg, 
+								const char *dir, struct network_config *n, GList **channel_keys)
 {
 	GList *gl;
 	GKeyFile *kf;
-	char *fn;
 	char **autojoin_list;
 	int autojoin_list_count;
 	
-	if (!n->keyfile) {
-		n->keyfile = g_key_file_new();
+	if (n->keyfile == NULL) {
+		n->keyfile = cfg->keyfile;
+		n->groupname = g_strdup(n->name);
 	}
 
 	kf = n->keyfile;
 
-	g_key_file_set_string(kf, "global", "fullname", n->fullname);
-	g_key_file_set_string(kf, "global", "nick", n->nick);
-	g_key_file_set_string(kf, "global", "username", n->username);
-	if (n->autocmd) {
-		g_key_file_set_string_list(kf, "global", "autocmd", n->autocmd,
+	if (n->fullname != NULL)
+		g_key_file_set_string(kf, n->groupname, "fullname", n->fullname);
+	else 
+		g_key_file_remove_key(kf, n->groupname, "fullname", NULL);
+	if (n->nick != NULL)
+		g_key_file_set_string(kf, n->groupname, "nick", n->nick);
+	else 
+		g_key_file_remove_key(kf, n->groupname, "nick", NULL);
+	if (n->username != NULL)
+		g_key_file_set_string(kf, n->groupname, "username", n->username);
+	else
+		g_key_file_remove_key(kf, n->groupname, "username", NULL);
+	if (n->autocmd != NULL) {
+		g_key_file_set_string_list(kf, n->groupname, "autocmd", (const char *const*)n->autocmd,
 							   g_strv_length(n->autocmd));
 	} else {
-		g_key_file_remove_key(kf, "global", "autocmd", NULL);
+		g_key_file_remove_key(kf, n->groupname, "autocmd", NULL);
 	}
 
 	if (n->queue_speed)
-		g_key_file_set_integer(kf, "global", "queue-speed", n->queue_speed);
+		g_key_file_set_integer(kf, n->groupname, "queue-speed", n->queue_speed);
 	if (n->reconnect_interval != -1)
-		g_key_file_set_integer(kf, "global", "reconnect-interval", n->reconnect_interval);
+		g_key_file_set_integer(kf, n->groupname, "reconnect-interval", n->reconnect_interval);
 
 	switch(n->type) {
 	case NETWORK_VIRTUAL:
-		g_key_file_set_string(kf, "global", "virtual", n->type_settings.virtual_type);
+		g_key_file_set_string(kf, n->groupname, "virtual", n->type_settings.virtual_type);
 		break;
 	case NETWORK_PROGRAM:
-		g_key_file_set_string(kf, "global", "program", n->type_settings.program_location);
+		g_key_file_set_string(kf, n->groupname, "program", n->type_settings.program_location);
 		break;
 	case NETWORK_TCP:
-		config_save_tcp_servers(n, kf);
+		config_save_tcp_servers(n, kf, n->groupname);
 		if (n->type_settings.tcp.default_bind_address != NULL)
-			g_key_file_set_string(kf, "global", "bind", 
+			g_key_file_set_string(kf, n->groupname, "bind", 
 								  n->type_settings.tcp.default_bind_address);
 		else
-			g_key_file_remove_key(kf, "global", "bind", NULL);
+			g_key_file_remove_key(kf, n->groupname, "bind", NULL);
 		break;
 	default:break;
 	}
@@ -242,16 +256,16 @@ static void config_save_network(const char *dir, struct network_config *n, GList
 	}
 
 	if (autojoin_list == NULL)
-		g_key_file_remove_key(kf, "global", "autojoin", NULL);
+		g_key_file_remove_key(kf, n->groupname, "autojoin", NULL);
 	else
-		g_key_file_set_string_list(kf, "global", "autojoin", (const gchar **)autojoin_list, 
+		g_key_file_set_string_list(kf, n->groupname, "autojoin", (const gchar **)autojoin_list, 
 							   autojoin_list_count);
 
 	g_free(autojoin_list);
 
-	fn = g_build_filename(dir, n->name, NULL);
-	g_key_file_save_to_file(kf, fn, NULL);
-	g_free(fn);
+	if (n->filename != NULL) {
+		g_key_file_save_to_file(kf, n->filename, NULL);
+	}
 }
 
 static void config_save_listeners(struct ctrlproxy_config *cfg, const char *path)
@@ -261,9 +275,9 @@ static void config_save_listeners(struct ctrlproxy_config *cfg, const char *path
 	GKeyFile *kf; 
 	GError *error = NULL;
 	gboolean empty = TRUE;
-	char *default_password;
 
-	default_password = g_key_file_get_string(cfg->keyfile, "global", "password", NULL);
+	if (cfg->password != NULL)
+		g_key_file_set_string(cfg->keyfile, "global", "password", cfg->password);
 
 	if (cfg->auto_listener) {
 		g_key_file_set_boolean(cfg->keyfile, "global", "listener-auto", cfg->auto_listener);
@@ -290,6 +304,7 @@ static void config_save_listeners(struct ctrlproxy_config *cfg, const char *path
 			if (l->network != NULL)
 				g_key_file_set_string(cfg->keyfile, "global", "default-network",
 								  l->network);
+			cfg->default_listener = l;
 		} else {
 			char *tmp;
 			empty = FALSE;
@@ -298,8 +313,7 @@ static void config_save_listeners(struct ctrlproxy_config *cfg, const char *path
 			else
 				tmp = g_strdup_printf("%s:%s", l->address, l->port);
 
-			if (l->password != NULL && 
-				!(default_password != NULL && strcmp(l->password, default_password) == 0)) 
+			if (l->password != NULL)
 				g_key_file_set_string(kf, tmp, "password", l->password);
 
 			if (l->network != NULL) {
@@ -307,6 +321,16 @@ static void config_save_listeners(struct ctrlproxy_config *cfg, const char *path
 			}
 
 			g_key_file_set_boolean(kf, tmp, "ssl", l->ssl);
+
+			if (l->ssl) {
+				if (l->certfile != NULL) {
+					g_key_file_set_string(kf, tmp, "certfile", l->certfile);
+				}
+				if (l->keyfile != NULL) {
+					g_key_file_set_string(kf, tmp, "keyfile", l->keyfile);
+				}
+			}
+
 		
 			g_free(tmp);
 		}
@@ -320,7 +344,6 @@ static void config_save_listeners(struct ctrlproxy_config *cfg, const char *path
 		}
 	}
 	
-	g_free(default_password);
 	g_free(filename);
 }
 
@@ -330,17 +353,18 @@ static void config_save_networks(struct ctrlproxy_config *cfg, const char *confi
 	GList *gl;
 	GList *channel_keys = NULL;
 
-	if (!g_file_test(networksdir, G_FILE_TEST_IS_DIR)) {
-		if (g_mkdir(networksdir, 0700) != 0) {
-			log_global(LOG_ERROR, "Can't create networks directory '%s': %s", networksdir, strerror(errno));
-			return;
-		}
-	}
-
 	for (gl = networks; gl; gl = gl->next) {
 		struct network_config *n = gl->data;
+		if (n->filename != NULL && !g_file_test(networksdir, G_FILE_TEST_IS_DIR)) {
+			if (g_mkdir(networksdir, 0700) != 0) {
+				log_global(LOG_ERROR, "Can't create networks directory '%s': %s", networksdir, strerror(errno));
+				return;
+			}
+		}
+
+
 		if (!n->implicit) 
-			config_save_network(networksdir, n, &channel_keys);
+			config_save_network(cfg, networksdir, n, &channel_keys);
 	}
 
 	config_cleanup_networks_dir(cfg);
@@ -392,9 +416,12 @@ void save_configuration(struct ctrlproxy_config *cfg, const char *configuration_
 		!cfg->admin_log)
 		g_key_file_set_boolean(cfg->keyfile, "global", "admin-log", cfg->admin_log);
 
-	if (g_key_file_has_key(cfg->keyfile, "global", "max_who_age", NULL) ||
-		cfg->max_who_age != 0)
-		g_key_file_set_integer(cfg->keyfile, "global", "max_who_age", cfg->max_who_age);
+	if (g_key_file_has_key(cfg->keyfile, "global", "max_who_age", NULL)) {
+		g_key_file_set_integer(cfg->keyfile, "global", "max_who_age", cfg->cache.max_who_age);
+	} else if (g_key_file_has_key(cfg->keyfile, "global", "max-who-age", NULL) ||
+			 cfg->cache.max_who_age != 0) {
+		g_key_file_set_integer(cfg->keyfile, "global", "max-who-age", cfg->cache.max_who_age);
+	}
 
 	if (g_key_file_has_key(cfg->keyfile, "global", "learn-nickserv", NULL) ||
 		!cfg->learn_nickserv)
@@ -432,6 +459,19 @@ void save_configuration(struct ctrlproxy_config *cfg, const char *configuration_
 		break;
 	}
 
+	if (cfg->default_realname != NULL)
+		g_key_file_set_string(cfg->keyfile, "global", "default-fullname", cfg->default_realname);
+	else 
+		g_key_file_remove_key(cfg->keyfile, "global", "default-fullname", NULL);
+	if (cfg->default_nick != NULL)
+		g_key_file_set_string(cfg->keyfile, "global", "default-nick", cfg->default_nick);
+	else 
+		g_key_file_remove_key(cfg->keyfile, "global", "default-nick", NULL);
+	if (cfg->default_username != NULL)
+		g_key_file_set_string(cfg->keyfile, "global", "default-username", cfg->default_username);
+	else
+		g_key_file_remove_key(cfg->keyfile, "global", "default-username", NULL);
+
 	if (cfg->report_time_offset != 0 ||
 		g_key_file_has_key(cfg->keyfile, "global", "report-time-offset", NULL))
 		g_key_file_set_integer(cfg->keyfile, "global", "report-time-offset", cfg->report_time_offset);
@@ -485,7 +525,7 @@ static void config_load_servers(struct network_config *n)
 	char **servers;
 	int i;
 	
-	servers = g_key_file_get_string_list(n->keyfile, "global", "servers", &size, NULL);
+	servers = g_key_file_get_string_list(n->keyfile, n->groupname, "servers", &size, NULL);
 
 	if (!servers)
 		return;
@@ -529,110 +569,87 @@ static struct channel_config *config_find_add_channel(struct network_config *nc,
 	return cc;
 }
 
-static struct network_config *config_load_network(struct ctrlproxy_config *cfg, const char *dirname, 
-												  const char *name, GList *channel_keys)
+static struct network_config *config_load_network_keyfile_group(struct ctrlproxy_config *cfg, 
+																const char *networkname,
+																GKeyFile *kf, 
+																const char *groupname, GList *channel_keys)
 {
-	GKeyFile *kf;
 	struct network_config *n;
-	char *filename;
 	GList *gl;
-	int i;
-	char **groups;
-	GError *error = NULL;
 	gsize size;
 
-	kf = g_key_file_new();
-
-	filename = g_build_filename(dirname, name, NULL);
-
-	if (!g_key_file_load_from_file(kf, filename, G_KEY_FILE_KEEP_COMMENTS, &error)) {	
-		log_global(LOG_ERROR, "Can't parse configuration file '%s': %s", filename, error->message);
-		g_free(filename);
-		g_key_file_free(kf);
-		return NULL;
-	}	
-
 	n = network_config_init(cfg);
 	n->keyfile = kf;
+	n->groupname = g_strdup(groupname);
 
-	g_free(filename);
-
-	if (g_key_file_has_key(kf, "global", "fullname", NULL)) {
+	if (g_key_file_has_key(kf, groupname, "fullname", NULL)) {
 		g_free(n->fullname);
-		n->fullname = g_key_file_get_string(kf, "global", "fullname", NULL);
+		n->fullname = g_key_file_get_string(kf, groupname, "fullname", NULL);
 		if (!strcmp(n->fullname, "") || n->fullname[0] == ' ')
 			log_global(LOG_WARNING, "Invalid fullname `%s' set for network `%s'", n->fullname, n->name);
 	}
 
-	if (g_key_file_has_key(kf, "global", "nick", NULL)) {
+	if (g_key_file_has_key(kf, groupname, "nick", NULL)) {
 		g_free(n->nick);
-		n->nick = g_key_file_get_string(kf, "global", "nick", NULL);
+		n->nick = g_key_file_get_string(kf, groupname, "nick", NULL);
 		if (!strcmp(n->nick, "") || n->nick[0] == ' ')
 			log_global(LOG_WARNING, "Invalid nick name `%s' set for `%s'", n->nick, n->name);
 	}
 
-	if (g_key_file_has_key(kf, "global", "reconnect-interval", NULL)) {
-		n->reconnect_interval = g_key_file_get_integer(kf, "global", "reconnect-interval", NULL);
+	if (g_key_file_has_key(kf, groupname, "reconnect-interval", NULL)) {
+		n->reconnect_interval = g_key_file_get_integer(kf, groupname, "reconnect-interval", NULL);
 	}
 
-	if (g_key_file_has_key(kf, "global", "queue-speed", NULL)) {
-		n->queue_speed = g_key_file_get_integer(kf, "global", "queue-speed", NULL);
+	if (g_key_file_has_key(kf, groupname, "queue-speed", NULL)) {
+		n->queue_speed = g_key_file_get_integer(kf, groupname, "queue-speed", NULL);
 	}
 
-	if (g_key_file_has_key(kf, "global", "username", NULL)) {
+	if (g_key_file_has_key(kf, groupname, "username", NULL)) {
 		g_free(n->username);
-		n->username = g_key_file_get_string(kf, "global", "username", NULL);
+		n->username = g_key_file_get_string(kf, groupname, "username", NULL);
 		if (!strcmp(n->username, "") || n->username[0] == ' ')
 			log_global(LOG_WARNING, "Invalid username `%s' set for network `%s'", n->username, n->name);
 	}
 
-	if (g_key_file_has_key(kf, "global", "ignore_first_nick", NULL)) {
-		n->ignore_first_nick = g_key_file_get_boolean(kf, "global", "ignore_first_nick", NULL);
+	if (g_key_file_has_key(kf, groupname, "ignore_first_nick", NULL)) {
+		n->ignore_first_nick = g_key_file_get_boolean(kf, groupname, "ignore_first_nick", NULL);
 	}
 
-	if (g_key_file_has_key(kf, "global", "password", NULL)) {
+	if (g_key_file_has_key(kf, groupname, "password", NULL)) {
 		g_free(n->password);
-		n->password = g_key_file_get_string(kf, "global", "password", NULL);
+		n->password = g_key_file_get_string(kf, groupname, "password", NULL);
 	}
 
-	if (g_key_file_has_key(kf, "global", "autocmd", NULL)) {
+	if (g_key_file_has_key(kf, groupname, "autocmd", NULL)) {
 		g_strfreev(n->autocmd);
-		n->autocmd = g_key_file_get_string_list(n->keyfile, "global", "autocmd", &size, NULL);
+		n->autocmd = g_key_file_get_string_list(n->keyfile, groupname, "autocmd", &size, NULL);
 	}
 
-	n->name = g_strdup(name);
+	n->name = g_strdup(networkname);
 
-	if (g_key_file_has_key(kf, "global", "program", NULL)) 
+	if (g_key_file_has_key(kf, groupname, "program", NULL)) 
 		n->type = NETWORK_PROGRAM;
-	else if (g_key_file_has_key(kf, "global", "virtual", NULL)) 
+	else if (g_key_file_has_key(kf, groupname, "virtual", NULL)) 
 		n->type = NETWORK_VIRTUAL;
 	else 
 		n->type = NETWORK_TCP;
 
 	switch (n->type) {
 	case NETWORK_TCP:
-		n->type_settings.tcp.default_bind_address = g_key_file_get_string(kf, "global", "bind", NULL);
+		n->type_settings.tcp.default_bind_address = g_key_file_get_string(kf, groupname, "bind", NULL);
 		config_load_servers(n);
 		break;
 	case NETWORK_PROGRAM:
-		n->type_settings.program_location = g_key_file_get_string(kf, "global", "program", NULL);
+		n->type_settings.program_location = g_key_file_get_string(kf, groupname, "program", NULL);
 		break;
 	case NETWORK_VIRTUAL:
-		n->type_settings.virtual_type = g_key_file_get_string(kf, "global", "virtual", NULL);
+		n->type_settings.virtual_type = g_key_file_get_string(kf, groupname, "virtual", NULL);
 		break;
 	case NETWORK_IOCHANNEL:
 		/* Don't store */
 		break;
 	}
 
-	groups = g_key_file_get_groups(kf, &size);
-	for (i = 0; i < size; i++) {
-		if (!g_ascii_isalpha(groups[i][0]))
-			config_load_channel(n, kf, groups[i]);
-	}
-
-	g_strfreev(groups);
-
 	for (gl = channel_keys; gl; gl = gl->next) {
 		struct keyfile_entry *ke = gl->data;
 
@@ -644,9 +661,10 @@ static struct network_config *config_load_network(struct ctrlproxy_config *cfg,
 		}
 	}
 
-	if (g_key_file_has_key(n->keyfile, "global", "autojoin", NULL)) {
+	if (g_key_file_has_key(n->keyfile, groupname, "autojoin", NULL)) {
 		char **autojoin_channels;
-		autojoin_channels = g_key_file_get_string_list(n->keyfile, "global", "autojoin", &size, NULL);
+		gsize i;
+		autojoin_channels = g_key_file_get_string_list(n->keyfile, groupname, "autojoin", &size, NULL);
 		for (i = 0; i < size; i++) {
 			struct channel_config *cc = config_find_add_channel(n, autojoin_channels[i]);
 
@@ -659,6 +677,42 @@ static struct network_config *config_load_network(struct ctrlproxy_config *cfg,
 	return n;
 }
 
+static struct network_config *config_load_network_file(struct ctrlproxy_config *cfg, const char *dirname, 
+												  const char *name, GList *channel_keys)
+{
+	GKeyFile *kf;
+	char *filename;
+	int i;
+	char **groups;
+	GError *error = NULL;
+	gsize size;
+	struct network_config *n;
+
+	kf = g_key_file_new();
+
+	filename = g_build_filename(dirname, name, NULL);
+
+	if (!g_key_file_load_from_file(kf, filename, G_KEY_FILE_KEEP_COMMENTS, &error)) {	
+		log_global(LOG_ERROR, "Can't parse configuration file '%s': %s", filename, error->message);
+		g_free(filename);
+		g_key_file_free(kf);
+		return NULL;
+	}	
+
+	n = config_load_network_keyfile_group(cfg, name, kf, "global", channel_keys);
+	n->filename = filename;
+
+	groups = g_key_file_get_groups(n->keyfile, &size);
+	for (i = 0; i < size; i++) {
+		if (!g_ascii_isalpha(groups[i][0]))
+			config_load_channel(n, n->keyfile, groups[i]);
+	}
+
+	g_strfreev(groups);
+
+	return n;
+}
+
 static struct network_config *find_create_network_config(struct ctrlproxy_config *cfg, const char *name)
 {
 	GList *gl;
@@ -762,7 +816,6 @@ static void config_load_listeners(struct ctrlproxy_config *cfg)
 	char **groups;
 	gsize size;
 	GKeyFile *kf;
-	char *default_password;
 	GError *error = NULL;
 
 	if (g_key_file_has_key(cfg->keyfile, "listener", "pasword", NULL)) {
@@ -770,7 +823,8 @@ static void config_load_listeners(struct ctrlproxy_config *cfg)
 							  g_key_file_get_string(cfg->keyfile, "listener", "password", NULL));
 		g_key_file_remove_key(cfg->keyfile, "listener", "password", NULL);
 	}
-	default_password = g_key_file_get_string(cfg->keyfile, "global", "password", NULL);
+	cfg->password = g_key_file_get_string(cfg->keyfile, "global", "password", NULL);
+
 	if (g_key_file_has_key(cfg->keyfile, "global", "listener-auto", NULL)) {
 		cfg->auto_listener = g_key_file_get_boolean(cfg->keyfile, "global", "listener-auto", NULL);
 	} else if (g_key_file_has_key(cfg->keyfile, "listener", "auto", NULL)) {
@@ -792,6 +846,18 @@ static void config_load_listeners(struct ctrlproxy_config *cfg)
 		l->address = g_key_file_get_string(cfg->keyfile, "global", "bind", NULL);
 		l->ssl = g_key_file_has_key(cfg->keyfile, "global", "ssl", NULL) &&
 				 g_key_file_get_boolean(cfg->keyfile, "global", "ssl", NULL);
+		
+#ifdef HAVE_GNUTLS
+		if (l->ssl) {
+			l->ssl_credentials = ssl_create_server_credentials(cfg->config_dir, cfg->keyfile, "global");
+			if (g_key_file_has_key(cfg->keyfile, "global", "certfile", NULL)) {
+				l->certfile = g_key_file_get_string(cfg->keyfile, "global", "certfile", NULL);
+			}
+			if (g_key_file_has_key(cfg->keyfile, "global", "keyfile", NULL)) {
+				l->keyfile = g_key_file_get_string(cfg->keyfile, "global", "keyfile", NULL);
+			}
+		}
+#endif
 		l->is_default = TRUE;
 
 		l->network = g_key_file_get_string(cfg->keyfile, "global", "default-network", NULL);
@@ -805,7 +871,6 @@ static void config_load_listeners(struct ctrlproxy_config *cfg)
 		if (error->code != G_FILE_ERROR_NOENT)
 			log_global(LOG_ERROR, "Can't parse configuration file '%s': %s", filename, error->message);
 		g_free(filename);
-		g_free(default_password);
 		return;
 	}
 		
@@ -830,15 +895,20 @@ static void config_load_listeners(struct ctrlproxy_config *cfg)
 		}
 
 		l->password = g_key_file_get_string(kf, groups[i], "password", NULL);
-		if (l->password == NULL)
-			l->password = g_strdup(default_password);
 
 		if (g_key_file_has_key(kf, groups[i], "ssl", NULL))
 			l->ssl = g_key_file_get_boolean(kf, groups[i], "ssl", NULL);
 
 #ifdef HAVE_GNUTLS
-		if (l->ssl)
-			l->ssl_credentials = ssl_create_server_credentials(cfg, kf, groups[i]);
+		if (l->ssl) {
+			l->ssl_credentials = ssl_create_server_credentials(cfg->config_dir, kf, groups[i]);
+			if (g_key_file_has_key(kf, groups[i], "certfile", NULL)) {
+				l->certfile = g_key_file_get_string(kf, groups[i], "certfile", NULL);
+			}
+			if (g_key_file_has_key(kf, groups[i], "keyfile", NULL)) {
+				l->keyfile = g_key_file_get_string(kf, groups[i], "keyfile", NULL);
+			}
+		}
 #endif
 
 		if (g_key_file_has_key(kf, groups[i], "network", NULL))
@@ -848,7 +918,6 @@ static void config_load_listeners(struct ctrlproxy_config *cfg)
 	}
 
 	g_strfreev(groups);
-	g_free(default_password);
 	g_free(filename);
 }
 
@@ -902,20 +971,42 @@ static void config_load_networks(struct ctrlproxy_config *cfg, GList *channel_ke
 	char *networksdir = g_build_filename(cfg->config_dir, "networks", NULL);
 	GDir *dir;
 	const char *name;
+	char **groups;
+	gsize size, i;
+	struct network_config *n;
+
+	/* Load networks from files */
 
 	dir = g_dir_open(networksdir, 0, NULL);
-	if (dir == NULL)
-		return;
+	if (dir != NULL) {
+		while ((name = g_dir_read_name(dir))) {
+			if (IS_SPECIAL_FILE(name))
+				continue;
+			n = config_load_network_file(cfg, networksdir, name, channel_keys);
+		}
 
-	while ((name = g_dir_read_name(dir))) {
-		if (IS_SPECIAL_FILE(name))
-			continue;
-		config_load_network(cfg, networksdir, name, channel_keys);
+		g_dir_close(dir);
 	}
-
 	g_free(networksdir);
 
-	g_dir_close(dir);
+
+	/* Load other networks in configuration file */
+	groups = g_key_file_get_groups(cfg->keyfile, &size);
+	for (i = 0; i < size; i++) {
+		if (!strcmp(groups[i], "global"))
+			continue;
+
+		if (!strncasecmp(groups[i], "irc://", strlen("irc://")) || 
+			!strncasecmp(groups[i], "ircs://", strlen("ircs://")))
+			continue;
+
+		if (groups[i][0] == '#' || groups[i][0] == '&')
+			continue;
+
+		n = config_load_network_keyfile_group(cfg, groups[i], cfg->keyfile, groups[i], channel_keys);
+	}
+
+	g_strfreev(groups);
 }
 
 #define FETCH_SETTING(data, kf, section, prefix, name) (data)->name = g_key_file_get_string((kf), (section), prefix __STRING(name), NULL)
@@ -1157,9 +1248,20 @@ struct ctrlproxy_config *load_configuration(const char *dir)
 		!g_key_file_get_boolean(kf, "global", "autosave", NULL))
 		cfg->autosave = FALSE;
 
+	if (g_key_file_has_key(kf, "global", "keytab", NULL)) {
+		char *keytab;
+		keytab = g_key_file_get_string(kf, "global", "keytab", NULL);
+#ifdef HAVE_GSSKRB5_REGISTER_ACCEPTOR_IDENTITY
+		gsskrb5_register_acceptor_identity(keytab);
+#endif
+		g_free(keytab);
+	}
 
 	if (g_key_file_has_key(kf, "global", "max_who_age", NULL))
-		cfg->max_who_age = g_key_file_get_integer(kf, "global", "max_who_age", NULL);
+		cfg->cache.max_who_age = g_key_file_get_integer(kf, "global", "max_who_age", NULL);
+	else if (g_key_file_has_key(kf, "global", "max-who-age", NULL))
+		cfg->cache.max_who_age = g_key_file_get_integer(kf, "global", "max-who-age", NULL);
+
 
 	cfg->replication = g_key_file_get_string(kf, "global", "replication", NULL);
 	cfg->linestack_backend = g_key_file_get_string(kf, "global", "linestack", NULL);
@@ -1187,7 +1289,7 @@ struct ctrlproxy_config *load_configuration(const char *dir)
     if (g_key_file_has_key(kf, "global", "motd-file", NULL))
 		cfg->motd_file = g_key_file_get_string(kf, "global", "motd-file", NULL);
     else 
-	    cfg->motd_file = g_build_filename(SHAREDIR, "motd", NULL);
+	    cfg->motd_file = g_build_filename(SYSCONFDIR, "ctrlproxy", "motd", NULL);
 
     if (g_key_file_has_key(kf, "client", "charset", NULL)) {
 		cfg->client_charset = g_key_file_get_string(kf, "client", "charset", NULL);
@@ -1216,6 +1318,27 @@ struct ctrlproxy_config *load_configuration(const char *dir)
 	else
 		cfg->create_implicit = TRUE;
 
+	if (g_key_file_has_key(kf, "global", "default-username", NULL)) {
+		g_free(cfg->default_username);
+		cfg->default_username = g_key_file_get_string(kf, "global", "default-username", NULL);
+		if (!strcmp(cfg->default_username, "") || cfg->default_username[0] == ' ')
+			log_global(LOG_WARNING, "Invalid default username `%s' set", cfg->default_username);
+	}
+
+	if (g_key_file_has_key(kf, "global", "default-fullname", NULL)) {
+		g_free(cfg->default_realname);
+		cfg->default_realname = g_key_file_get_string(kf, "global", "default-fullname", NULL);
+		if (!strcmp(cfg->default_realname, "") || cfg->default_realname[0] == ' ')
+			log_global(LOG_WARNING, "Invalid default fullname `%s' set", cfg->default_realname);
+	}
+
+	if (g_key_file_has_key(kf, "global", "default-nick", NULL)) {
+		g_free(cfg->default_nick);
+		cfg->default_nick = g_key_file_get_string(kf, "global", "default-nick", NULL);
+		if (!strcmp(cfg->default_nick, "") || cfg->default_nick[0] == ' ')
+			log_global(LOG_WARNING, "Invalid nick name `%s' set", cfg->default_nick);
+	}
+
 	if (!g_file_test(cfg->motd_file, G_FILE_TEST_EXISTS))
 		log_global(LOG_ERROR, "Can't open MOTD file '%s' for reading", cfg->motd_file);
 
@@ -1238,7 +1361,6 @@ struct ctrlproxy_config *load_configuration(const char *dir)
 	g_key_file_remove_key(kf, "admin", "log", NULL);
     if (g_key_file_has_key(kf, "global", "admin-log", NULL) && !g_key_file_get_boolean(kf, "global", "admin-log", NULL))
         cfg->admin_log = FALSE;
-	g_key_file_remove_group(kf, "admin", NULL);
 
 	for (gl = cfg->networks; gl; gl = gl->next) {
 		struct network_config *nc = gl->data;
@@ -1293,16 +1415,9 @@ struct network_config *network_config_init(struct ctrlproxy_config *cfg)
 {
 	struct network_config *s = g_new0(struct network_config, 1);
 
+	s->global = cfg;
+
 	s->autoconnect = FALSE;
-	s->nick = g_strdup(g_get_user_name());
-	s->username = g_strdup(g_get_user_name());
-	g_assert(s->username != NULL && strlen(s->username) > 0);
-	s->fullname = g_strdup(g_get_real_name());
-	if (s->fullname == NULL || 
-		strlen(s->fullname) == 0) {
-		g_free(s->fullname);
-		s->fullname = g_strdup(s->username);
-	}
 	s->reconnect_interval = -1;
 
 	if (cfg) 
@@ -1351,7 +1466,10 @@ void free_config(struct ctrlproxy_config *cfg)
 			break;
 		}
 		cfg->networks = g_list_remove(cfg->networks, nc);
-		if (nc->keyfile) g_key_file_free(nc->keyfile);
+		if (nc->keyfile != NULL && nc->filename != NULL) 
+			g_key_file_free(nc->keyfile);
+		g_free(nc->filename);
+		g_free(nc->groupname);
 		g_free(nc);
 	}
 	g_free(cfg->config_dir);
@@ -1413,6 +1531,8 @@ gboolean create_configuration(const char *config_dir)
 		l->password = g_strdup(pass);
 	}
 
+	global->config->default_listener = l;
+
 	global->config->listeners = g_list_append(global->config->listeners, l);
 
 	save_configuration(global->config, config_dir);
